/**
 * Copyright (C) 2010-2013 Jakub Skokan <aither@havefun.cz>
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3
 * as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <amxmodx>
#include <amxmisc>
#include <dodx>
#include <sqlx>
#include <dod_ranks>
#include <hf_utils>

#define RANK_CORRECTION
#define OVERRIDE_DOD_STATS
//#define SHOW_POINTS_HUD

#if defined RANK_CORRECTION
#include <time>
#endif

#define VERSION "0.12.0-dev"
// #define pratio(%1) ((g_players[(%1)][DEATHS] == 0) ? float(g_players[(%1)][KILLS]) : (float(g_players[(%1)][KILLS]) / float(g_players[(%1)][DEATHS])))
// #define mratio(%1) ((g_players[(%1)][KILLS] == 0) ? float(g_players[(%1)][DEATHS]) : (float(g_players[(%1)][DEATHS]) / float(g_players[(%1)][KILLS])))
#define kpm(%1) (float(g_players[(%1)][KILLS]) / float(g_players[(%1)][GAME_TIME] + get_user_time((%1), 1)) * 60.0)
#define dpm(%1) (float(g_players[(%1)][DEATHS]) / float(g_players[(%1)][GAME_TIME] + get_user_time((%1), 1)) * 60.0)

enum _:Ranks {
	FIRST_RANK = 0,
	RECRUIT = FIRST_RANK,
	PRIVATE,
	PRIVATE_FIRST_CLASS,
	CORPORAL,
	SERGEANT,
	STAFF_SERGEANT,
	MASTER_SERGEANT,
	SECOND_LIUTENANT,
	FIRST_LIUTENANT,
	CAPTAIN,
	MAJOR,
	LIUTENANT_COLONEL,
	COLONEL,
	GENERAL,
	LAST_RANK = GENERAL,
	RANKS_COUNT,
}

new const RANKSTRINGS[RANKS_COUNT][] = 
{ 
	"RECRUIT",
	"PRIVATE",
	"PRIVATE_FIRST_CLASS",
	"CORPORAL",
	"SERGEANT",
	"STAFF_SERGEANT",
	"MASTER_SERGEANT",
	"SECOND_LIUTENANT",
	"FIRST_LIUTENANT",
	"CAPTAIN",
	"MAJOR",
	"LIUTENANT_COLONEL",
	"COLONEL",
	"GENERAL"
}

enum _:Players {
	RANK,
	POINTS,
	KILLS,
	DEATHS,
	SCORE,
	ORIGIN_KILLS,
	ORIGIN_DEATHS,
	ORIGIN_SCORE,
	NOTIFIED,
	PASSWORD,
	TYPE,
	LOADED,
	GAME_TIME,
	DB_ID,
}

enum {
	EVENT_KILL=0,
	EVENT_DEATH,
	EVENT_HS,
	EVENT_SCORE,
	EVENT_SUICIDE,
	EVENT_TK,
	EVENTS_COUNT,
	
	TYPE_NONSTEAM=0,
	TYPE_STEAM,
	TYPE_IGNORE,
	
	TASK_CONFIGURE=40,
	TASK_LOADED=50,
#if defined RANK_CORRECTION
	TASK_CORRECTION=100,
#endif
}

new g_players[33][Players]
new g_events[EVENTS_COUNT] = {0}
new g_rankpoints[RANKS_COUNT] = {0}

new Handle:g_SqlTuple
new Handle:g_connection = Empty_Handle
new
	g_cvar_admin_mask,
	g_cvar_admin_mask_level,
	g_cvar_load_rank,
	g_cvar_apply_ratio,
	g_cvar_apply_kpm
	
#if defined RANK_CORRECTION
new g_cvar_rank_correction_time
#endif

new g_loaded = 0
new g_game_id
new Float:g_server_weight

new g_buffer[2176]

public plugin_init()
{
	register_plugin("HF DoD Military Ranks", VERSION, "aither")
	register_dictionary("dod_ranks.txt")
	
	g_cvar_admin_mask = register_cvar("ranks_admin_mask", "1")
	g_cvar_admin_mask_level = register_cvar("ranks_admin_mask_level", "a")
	g_cvar_load_rank = register_cvar("ranks_load_rank", "1")
	g_cvar_apply_ratio = register_cvar("ranks_apply_ratio", "1")
	g_cvar_apply_kpm = register_cvar("ranks_apply_kpm", "1")
	
#if defined _time_included
	g_cvar_rank_correction_time = register_cvar("ranks_correction_time", "120")
	register_dictionary("time.txt")
#endif
	
	register_statsfwd(XMF_DEATH)
	register_statsfwd(XMF_SCORE)
	
#if defined OVERRIDE_DOD_STATS
	register_clcmd("say /rank", "cmd_myrank")
#endif
	
	register_clcmd("say /myrank", "cmd_myrank")
	register_clcmd("say /ranks", "cmd_ranks")
	register_clcmd("say /helpranks", "cmd_helpranks")
	register_clcmd("say /levels", "cmd_levels")
	register_concmd("ranks_list", "cmd_listranks", ADMIN_ALL, "- list players' ranks")
	register_concmd("ranks_setrank", "cmd_setrank", ADMIN_IMMUNITY, "<target> <rank> - set target's rank")
	register_concmd("ranks_setpoints", "cmd_setpoints", ADMIN_IMMUNITY, "<target> <points> - set target's points")
	register_concmd("ranks_addpoints", "cmd_addpoints", ADMIN_IMMUNITY, "<target> <points> - add points to target")
	register_concmd("ranks_removepoints", "cmd_removepoints", ADMIN_IMMUNITY, "<target> <points> - remove points from target")
	register_concmd("ranks_promote", "cmd_promote", ADMIN_IMMUNITY, "<target> - promotes player if possible")
	register_concmd("ranks_suspend", "cmd_suspend", ADMIN_IMMUNITY, "<target> - suspends player if possible")
	register_concmd("ranks_saveall", "cmd_saveall", ADMIN_IMMUNITY, "save ranks of all players to db")
	
	new i
	for(i = 1; i < 33; i++)
		reset_player(i)
}

public plugin_natives()
{
	register_native("ranks_get_info", "native_ranks_get_info")
}

public plugin_cfg()
{
	set_task(0.25, "configure", TASK_CONFIGURE)
}

public configure()
{
	if(g_connection == Empty_Handle)
	{
		new host[64], user[64], pass[64], db[64]
		get_cvar_string("amx_sql_host", host, 63)
		get_cvar_string("amx_sql_user", user, 63)
		get_cvar_string("amx_sql_pass", pass, 63)
		get_cvar_string("amx_sql_db", db, 63)
		
		g_SqlTuple = SQL_MakeDbTuple(host, user, pass, db)
		new error[512], errno
		g_connection = SQL_Connect(g_SqlTuple, errno, error, 511)
		
		if(g_connection == Empty_Handle)
		{
			// stop the plugin with an error message
			//set_fail_state(error)
			log_amx("Cannot connect to db, I'll try it again later.")
			set_task(30.0, "configure", TASK_CONFIGURE)
			
			return PLUGIN_CONTINUE
		}
	}
	
	new query[500], ip[16], port[6]
	
	get_cvar_string("ip", ip, 15)
	get_cvar_string("port", port, 5)
	
	formatex(query, 499, "SELECT g.id, g.name, gs.weight FROM ranks_game g INNER JOIN ranks_servergame gs ON g.id = gs.game_id INNER JOIN halflife_server s ON gs.server_id = s.id WHERE s.bind_ip = '%s' AND s.port = %s AND g.state = 1", ip, port)
	
	new Handle:query_game = SQL_PrepareQuery(g_connection, query)

	/*if( !SQL_Execute(query_game) )
	{
		new error[512]
		SQL_QueryError(query_game, error, 511)
		return set_fail_state(error)
	}*/
	if(!check_execute(query_game))
		return PLUGIN_CONTINUE
	
	if( SQL_NumResults(query_game) == 0 )
	{
		new error[100]
		formatex(error, 99, "This server is not a part of any war! I'll try it again later.")
			
		SQL_FreeHandle(query_game)
		
		//return set_fail_state(error)
		
		log_amx(error)
		set_task(30.0, "configure", TASK_CONFIGURE)
		
		return PLUGIN_CONTINUE
	}
	
	new game_name[21], server_weight
	
	g_game_id = SQL_ReadResult(query_game, 0)
	SQL_ReadResult(query_game, 1, game_name, 20)
	server_weight = SQL_ReadResult(query_game, 2)
	g_server_weight = float(server_weight) / float(100)
	
	log_amx("Joined war ^"%s^" with weight %d ^%", game_name, server_weight)
	
	SQL_FreeHandle(query_game)
	
	new i
	for(i = 0; i < EVENTS_COUNT; i++)
		g_events[i] = 0
	
	formatex(query, 199, "SELECT rank, points FROM ranks_rankconfig WHERE game_id = %d ORDER BY rank", g_game_id)
	
	new Handle:query_ranks = SQL_PrepareQuery(g_connection, query)

	if(!check_execute(query_ranks))
		return PLUGIN_CONTINUE
	
	while(SQL_MoreResults(query_ranks))
	{
		g_rankpoints[SQL_ReadResult(query_ranks, 0)] = SQL_ReadResult(query_ranks, 1)
		
		SQL_NextRow(query_ranks)
	}
	
	formatex(query, 199, "SELECT event, value FROM ranks_pointconfig WHERE game_id = %d", g_game_id)
	
	new Handle:query_points = SQL_PrepareQuery(g_connection, query)

	if(!check_execute(query_points))
		return PLUGIN_CONTINUE
	
	new event;
	while(SQL_MoreResults(query_points))
	{
		event = SQL_ReadResult(query_points, 0)
		
		if(event >= EVENTS_COUNT)
		{
			log_amx("Compatibility error: event %d not recognised", event)
			continue;
		}
		
		g_events[event] = SQL_ReadResult(query_points, 1)
		
		SQL_NextRow(query_points)
	}
	
	SQL_FreeHandle(query_points)
	SQL_FreeHandle(query_ranks)
	
	g_loaded = 1
	
	return PLUGIN_CONTINUE
}

public plugin_end()
{
	if(g_connection != Empty_Handle)
		SQL_FreeHandle(g_connection)
	
	SQL_FreeHandle(g_SqlTuple)
}

public hf_player_changed(id, db_id)
{
	save_player(id)
	hf_player_authenticated(id, db_id)
}

public hf_player_disconnected(id)
{
#if defined RANK_CORRECTION
	if( g_players[id][RANK] == FIRST_RANK )
		remove_task(TASK_CORRECTION + id)
#endif
	
	remove_task(id)
	
	save_player(id)
	reset_player(id)
	
	return PLUGIN_CONTINUE
}

public client_death(killer, victim, wpnindex, hitplace, TK)
{
	if(killer == victim)
	{
		if( (wpnindex == DODW_HANDGRENADE || wpnindex == DODW_STICKGRENADE || wpnindex == DODW_STICKGRENADE_EX || wpnindex == DODW_HANDGRENADE_EX || wpnindex == DODW_MILLS_BOMB) && g_players[killer][TYPE] != TYPE_IGNORE )
		{
			if( g_players[killer][KILLS] > 0 )
				g_players[killer][KILLS]--;
			g_players[killer][DEATHS]++;
		
			//client_print(victim, print_chat, "%L", victim, "SUICIDE")
			
#if defined RANK_CORRECTION
			if( g_players[victim][RANK] > FIRST_RANK )
#endif
			points_remove(victim, g_events[EVENT_SUICIDE])
		}
	} else if(TK) {
		if( g_players[killer][TYPE] != TYPE_IGNORE && get_cvar_num("mp_friendlyfire"))
		{
			//client_print(killer, print_chat, "%L", killer, "TK")

			if( g_players[killer][KILLS] > 0 )
				g_players[killer][KILLS]--;
			
#if defined RANK_CORRECTION
			if( g_players[killer][RANK] > FIRST_RANK )
#endif
			points_remove(killer, g_events[EVENT_TK])
		}
	} else {
		if( g_players[killer][TYPE] != TYPE_IGNORE )
		{
			g_players[killer][KILLS]++;
#if defined RANK_CORRECTION
			if( g_players[killer][RANK] > FIRST_RANK )
#endif
			points_add(killer, g_events[ hitplace == HIT_HEAD ? EVENT_HS : EVENT_KILL ])
		}
		if( g_players[victim][TYPE] != TYPE_IGNORE )
		{
			g_players[victim][DEATHS]++;
#if defined RANK_CORRECTION
			if( g_players[victim][RANK] > FIRST_RANK )
#endif
			points_remove(victim, g_events[EVENT_DEATH])
		}
	}
	
	return PLUGIN_CONTINUE
}

public client_score(id, score, total)
{
	if( g_players[id][TYPE] != TYPE_IGNORE )
	{
#if defined RANK_CORRECTION
		if( g_players[id][RANK] > FIRST_RANK )
#endif
		points_add(id, g_events[EVENT_SCORE])
		g_players[id][SCORE]++
	}
	
	return PLUGIN_CONTINUE
}

public cmd_myrank(id)
{
	if( !g_players[id][LOADED] )
	{
		client_print(id, print_chat, "%L", id, "RANK_NOT_LOADED")
		return PLUGIN_HANDLED
	}
	
#if defined RANK_CORRECTION
	if( g_players[id][RANK] == FIRST_RANK )
	{
		new length[41]
		get_time_length(id, get_pcvar_num(g_cvar_rank_correction_time) * 60 - (g_players[id][GAME_TIME] + get_user_time(id, 1)), timeunit_seconds, length, 40)
		client_print(id, print_chat, "%L", id, "YOUR_RANK_CORRECTION", id, RANKSTRINGS[ g_players[id][RANK] ], g_players[id][RANK]+1, length)
	} else
#endif
	client_print(id, print_chat, "%L", id, "YOUR_RANK", id, RANKSTRINGS[ g_players[id][RANK] ], g_players[id][RANK]+1, g_players[id][POINTS], points_to_next_level(id))
	
	if( get_pcvar_num(g_cvar_apply_ratio) )
		client_print(id, print_chat, "%L", id, "YOUR_RATIO", pratio(id), mratio(id))
	
	if( get_pcvar_num(g_cvar_apply_kpm) )
		client_print(id, print_chat, "%L", id, "YOUR_KPM", kpm(id), dpm(id))
	
	return PLUGIN_HANDLED
}

public cmd_ranks(id)
{
	new players[32], player_count, i, name[32]
	new flags_str[2]
	
	get_pcvar_string(g_cvar_admin_mask_level, flags_str, 1)
	
	new admin_level = read_flags( flags_str )
	
	get_players(players, player_count, "ch")
	
	new pos = copy(g_buffer, 2175, "<html><head><style>body{background-color:#000000;margin:0 0 0 8px;color:#FFB000;}</style></head><body><table>")
	pos += format(g_buffer[pos], 2175-pos, "<tr><th>#</th><th>%L</th><th>%L</th><th>%L</th></tr>",
		id, "PLAYER_NAME", id, "PLAYER_RANK", id, "PLAYER_POINTS")
	
	new pl_rank[RANK_MAX_LENGTH]
	
	for(i = 0; i < player_count; i++)
	{
		get_user_name(players[i], name, 31)
		
		if( g_players[ players[i] ][TYPE] == TYPE_IGNORE )
			formatex(pl_rank, RANK_MAX_LENGTH-1, "%L", id, "PLAYER_IGNORED")
		else if( get_pcvar_num(g_cvar_admin_mask) && players[i] != id && access(players[i], admin_level) )
			copy(pl_rank, RANK_MAX_LENGTH-1, "ADMIN_RANK")
		else
			copy(pl_rank, RANK_MAX_LENGTH-1, RANKSTRINGS[ g_players[ players[i] ][RANK] ] )
		
		pos += format(g_buffer[pos], 2175-pos, "<tr><td>%d.</td><td>%s</td><td>%L</td><td>%d</td></tr>",
			i+1, name, players[i],
			pl_rank,
			g_players[ players[i] ][POINTS])
	}
	
	pos += format(g_buffer[pos], 2175-pos, "</table></body></html>")
	
	new header[50]
	formatex(header, 49, "%L", id, "RANKS_HEADER")
	
	show_motd(id, g_buffer, header)
	
	return PLUGIN_HANDLED
}

public cmd_helpranks(id)
{
	new len = 2175
	new pos = copy(g_buffer, len, "<html><head><style>body{background-color:#000000;margin:0 0 0 8px;color:#FFB000;}</style></head><body>")
	
	pos += format(g_buffer[pos], len-pos, "<p>%L</p>", id, "ABOUT_RANK")
	
	if( get_pcvar_num(g_cvar_apply_ratio) )
		pos += format(g_buffer[pos], len-pos, "<p>%L</p>", id, "RATIO_EXPLANATION")
	
	if( get_pcvar_num(g_cvar_apply_kpm) )
		pos += format(g_buffer[pos], len-pos, "<p>%L</p>", id, "KPM_EXPLANATION")
	
#if defined RANK_CORRECTION
	new length[41]
	get_time_length(id, get_pcvar_num(g_cvar_rank_correction_time), timeunit_minutes, length, 40)
	
	pos += format(g_buffer[pos], len-pos, "<p>%L</p>", id, "RANK_CORRECTION", length)	
#endif
		
	pos += format(g_buffer[pos], len-pos, "<h4>%L</h4><ul><li>/myrank - %L</li><li>/ranks - %L</li><li>/levels - %L</li></ul></body></html>",
		id, "COMMANDS", id, "MYRANK", id, "RANKS", id, "LEVELS")
	
	new header[50]
	formatex(header, 49, "%L", id, "RANKS_OVERVIEW")
	
	show_motd(id, g_buffer, header)
	
	return PLUGIN_HANDLED
}

public cmd_levels(id, level, cid)
{
	new len = 2175
	new pos = copy(g_buffer, len, "<html><head><style>body{background-color:#000000;margin:0 0 0 8px;color:#FFB000;}</style></head><body><table>")
	
	pos += format(g_buffer[pos], len-pos, "<tr><th>#</th><th>%L</th><th>%L</th></tr>",
		id, "PLAYER_RANK", id, "PLAYER_POINTS")
	
	new i
	for(i = FIRST_RANK; i < RANKS_COUNT; i++)
		pos += format(g_buffer[pos], len-pos, "<tr><td>%d.</td><td>%L</td><td>%d</td></tr>",
			i+1, id, RANKSTRINGS[i], g_rankpoints[i])
	
	pos += format(g_buffer[pos], len-pos, "</table><dl><dt>%L:</dt><dd>%d</dd><dt>%L:</dt><dd>%d</dd><dt>%L:</dt><dd>%d</dd><dt>%L:</dt><dd>%d</dd><dt>%L:</dt><dd>%d</dd><dt>%L:</dt><dd>%d</dd></dl>",
		id, "POINTS_KILL", g_events[EVENT_KILL],
		id, "POINTS_HS", g_events[EVENT_HS],
		id, "POINTS_SCORE", g_events[EVENT_SCORE],
		id, "POINTS_DEATH", g_events[EVENT_DEATH],
		id, "POINTS_SUICIDE", g_events[EVENT_SUICIDE],
		id, "POINTS_TK", g_events[EVENT_TK])
	
	new header[50]
	formatex(header, 49, "%L", id, "RANKS_OVERVIEW")
	
	show_motd(id, g_buffer, header)
	
	return PLUGIN_HANDLED
}

public cmd_listranks(id, level, cid)
{
	if (!cmd_access(id, level, cid, 0))
		return PLUGIN_HANDLED
	
	new players[32], playersnum, i, name[32], rank_str[RANK_MAX_LENGTH]
	get_players(players, playersnum, "ch")
	
	if( playersnum <= 0 )
	{
		console_print(id, "No clients connected")
		return PLUGIN_HANDLED
	}
	
	console_print(id, " #  %-25.25s %-25.25s %6.6s", "nick", "rank", "points")
	for(i = 0; i < playersnum; i++)
	{
		get_user_name(players[i], name, 31)
		formatex(rank_str, RANK_MAX_LENGTH-1, "%L", id, RANKSTRINGS[ g_players[ players[i] ][RANK] ])
		
		console_print(id, "%2d  %-25.25s %-25.25s %6d", i+1, name, rank_str, g_players[ players[i] ][POINTS])
	}
	
	return PLUGIN_HANDLED
}

public cmd_setrank(id, level, cid)
{
	if (!cmd_access(id, level, cid, 2))
		return PLUGIN_HANDLED
	
	new arg1[24], arg2[10]
	read_argv(1, arg1, 23)
	read_argv(2, arg2, 9)
	
	new player = cmd_target(id, arg1, 2)
        if( !player )
		return PLUGIN_HANDLED
	
	new new_rank = str_to_num(arg2)
	
	if(new_rank < FIRST_RANK) new_rank = FIRST_RANK
	if(new_rank > LAST_RANK) new_rank = LAST_RANK
	
	g_players[player][RANK] = new_rank
	g_players[player][POINTS] = g_rankpoints[new_rank]
	
	new admin_name[32]
	get_user_name(id, admin_name, 31)

	client_print(id, print_chat, "%L", player, "ADMIN_RANK_SET", admin_name, player, RANKSTRINGS[new_rank])
	
	return PLUGIN_HANDLED
}

public cmd_setpoints(id, level, cid)
{
	if (!cmd_access(id, level, cid, 2))
		return PLUGIN_HANDLED
	
	new arg1[24], arg2[10]
	read_argv(1, arg1, 23)
	read_argv(2, arg2, 9)
	
	new player = cmd_target(id, arg1, 2)
        if( !player )
		return PLUGIN_HANDLED
	
	new new_points = str_to_num(arg2)
	
	if(new_points < 0) new_points = 0
	//if(new_points > g_rankpoints[LAST_RANK]) new_points = g_rankpoints[LAST_RANK]
	
	g_players[player][POINTS] = new_points
	g_players[player][RANK] = rank_from_points(new_points)
	
	new admin_name[32]
	get_user_name(id, admin_name, 31)

	client_print(id, print_chat, "%L", player, "ADMIN_POINTS_SET", admin_name, new_points)
	
	return PLUGIN_HANDLED
}

public cmd_addpoints(id, level, cid)
{
	if (!cmd_access(id, level, cid, 2))
		return PLUGIN_HANDLED
	
	new arg1[24], arg2[10]
	read_argv(1, arg1, 23)
	read_argv(2, arg2, 9)
	
	new player = cmd_target(id, arg1, 2)
        if( !player )
		return PLUGIN_HANDLED
	
	new points = str_to_num(arg2)
	
	if(points < 0)
		return PLUGIN_HANDLED
	
	new admin_name[32]
	get_user_name(id, admin_name, 31)

	client_print(id, print_chat, "%L", player, "ADMIN_POINTS_ADD", admin_name, points)
	
	points_add(id, points, 1, 0)
	
	return PLUGIN_HANDLED
}

public cmd_removepoints(id, level, cid)
{
	if (!cmd_access(id, level, cid, 2))
		return PLUGIN_HANDLED
	
	new arg1[24], arg2[10]
	read_argv(1, arg1, 23)
	read_argv(2, arg2, 9)
	
	new player = cmd_target(id, arg1, 2)
        if( !player )
		return PLUGIN_HANDLED
	
	new points = str_to_num(arg2)
	
	if(points < 0)
		return PLUGIN_HANDLED
	
	new admin_name[32]
	get_user_name(id, admin_name, 31)

	client_print(id, print_chat, "%L", player, "ADMIN_POINTS_REMOVE", admin_name, points)
	
	points_remove(id, points * -1, 0, 1)
	
	return PLUGIN_HANDLED
}

public cmd_promote(id, level, cid)
{
	if (!cmd_access(id, level, cid, 2))
		return PLUGIN_HANDLED
	
	new arg1[24]
	read_argv(1, arg1, 23)
	
	new player = cmd_target(id, arg1, 2)
	
	if( !player )
		return PLUGIN_HANDLED
	
	new admin_name[32]
	get_user_name(id, admin_name, 31)
	
	client_print(player, print_chat, "%L", player, "ADMIN_RANK_PROMOTE", admin_name)
	upgrade_player(player)
	
	return PLUGIN_HANDLED
}

public cmd_suspend(id, level, cid)
{
	if (!cmd_access(id, level, cid, 2))
		return PLUGIN_HANDLED
	
	new arg1[24]
	read_argv(1, arg1, 23)
	
	new player = cmd_target(id, arg1, 2)
	
	if( !player )
		return PLUGIN_HANDLED

	new admin_name[32]
	get_user_name(id, admin_name, 31)

	client_print(player, print_chat, "%L", player, "ADMIN_RANK_SUSPEND", admin_name)
	degrade_player(player)
	
	return PLUGIN_HANDLED
}

public cmd_saveall(id, level, cid)
{
	if (!cmd_access(id, level, cid, 0))
		return PLUGIN_HANDLED
	
	new players[32], playersnum, i, saved = 0
	get_players(players, playersnum, "ch")
	
	for(i = 0; i < playersnum; i++)
		if(save_player(players[i]))
			saved += 1
	
	console_print(id, "%d players saved", saved)
	
	return PLUGIN_HANDLED
}

public CommonQueryHandle(failstate, Handle:query, error[], errno, data[], datasize)
{
	if(failstate == TQUERY_CONNECT_FAILED)
       		return set_fail_state("Could not connect to SQL database.")
	else if(failstate == TQUERY_QUERY_FAILED)
	{
		log_amx("Error on query: %s", error)
		return set_fail_state("Query failed.")
	}
	
	if(errno)
		return log_amx("Error on query: %s", error)
	
	return PLUGIN_CONTINUE
}

public hf_player_authenticated(id, db_id)
{
	reset_player(id)
	g_players[id][DB_ID] = db_id
	
	if( !g_loaded )
	{
		set_task(10.0, "check_loaded", TASK_LOADED + id)
		return PLUGIN_CONTINUE
	}
	
	new name[32]
	get_user_info(id, "name", name,31) 
	
	mysql_escape_string(name, 31)
	new query[300]
	
	formatex(query, 299, "SELECT kills, deaths, rank, points, game_time FROM ranks_playergame WHERE game_id = %d AND player_id = %d", g_game_id, db_id)
	//log_amx(query)
	new Handle:query_pg = SQL_PrepareQuery(g_connection, query)
	
	if(!check_execute(query_pg))
	{
		set_task(30.0, "check_loaded", TASK_LOADED + id)
		return PLUGIN_CONTINUE
	}
	
	if( SQL_NumResults(query_pg) == 0 )
	{
		if(!load_points_from_stats(id, db_id))
			return PLUGIN_CONTINUE
	} else {
		g_players[id][KILLS] += SQL_ReadResult(query_pg, 0)
		g_players[id][DEATHS] += SQL_ReadResult(query_pg, 1)
		g_players[id][RANK] = SQL_ReadResult(query_pg, 2)
		g_players[id][POINTS] += SQL_ReadResult(query_pg, 3)
		g_players[id][GAME_TIME] = SQL_ReadResult(query_pg, 4)
		g_players[id][SCORE] = 0
	}
	
	g_players[id][ORIGIN_KILLS] = g_players[id][KILLS]
	g_players[id][ORIGIN_DEATHS] = g_players[id][DEATHS]
	g_players[id][ORIGIN_SCORE] = g_players[id][SCORE]
	g_players[id][NOTIFIED] = g_rankpoints[LAST_RANK]
	
	if( g_players[id][GAME_TIME] <= 0 )
		g_players[id][GAME_TIME] = 1
	
	set_task(20.0, "cmd_myrank", id)
	
	g_players[id][LOADED] = 1
	
#if defined RANK_CORRECTION
	check_rank_correction(id)
#endif
	//log_amx("manage_player: Player loaded %d - name=%s, authid=%s, ip=%s, kills=%d, deaths=%d, points=%d, rank=%d", id, name, authid, ip, g_players[id][KILLS], g_players[id][DEATHS], g_players[id][POINTS], g_players[id][RANK])
	return PLUGIN_CONTINUE
}

save_player(id)
{
	if( !g_players[id][LOADED] )
		return PLUGIN_CONTINUE
	
	new query[1024], name[32]
	get_user_name(id, name, 31)
	mysql_escape_string(name, 31)
	
	new authid[33], ip[16]
	get_user_authid(id, authid, 32)
	get_user_ip(id, ip, 15, 1)
	
	new kills = g_players[id][KILLS] - g_players[id][ORIGIN_KILLS]
	new deaths = g_players[id][DEATHS] - g_players[id][ORIGIN_DEATHS]
	new score = g_players[id][SCORE] - g_players[id][ORIGIN_SCORE]
	new time = get_user_time(id, 1)
	new pos = 0
	new len = 1023
	
	pos += copy(query, len-pos, "UPDATE ranks_player p INNER JOIN ranks_playergame pg ON p.id = pg.player_id SET ")
	pos += formatex(query[pos], len-pos, "pg.points = %d, pg.rank = %d, pg.last_activity = NOW(), pg.game_time = pg.game_time + %d, pg.kills = pg.kills + %d, pg.deaths = pg.deaths + %d, pg.score = pg.score + %d, pg.pratio = %f, pg.mratio = %f, pg.kpm = %f, pg.dpm = %f, p.kills = p.kills + %d, p.deaths = p.deaths + %d, p.score = p.score + %d, p.total_game_time = p.total_game_time + %d ",
			g_players[id][POINTS], g_players[id][RANK], time, kills, deaths, score, pratio(id), mratio(id), kpm(id), dpm(id),
			kills, deaths, score, time)
	pos += formatex(query[pos], len-pos, ", p.pratio = p.kills / IFNULL(p.deaths, 1), p.mratio = p.deaths / IFNULL(p.kills, 1), p.kpm = p.kills / p.total_game_time * 60, p.dpm = p.deaths / p.total_game_time * 60 ")
	
	if( g_players[id][TYPE] == TYPE_STEAM ) {
		pos += formatex(query[pos], len-pos, ", p.name = '%s' WHERE player_id = %d AND pg.game_id = %d", name, g_players[id][DB_ID], g_game_id)
	} else {
		pos += formatex(query[pos], len-pos, "WHERE player_id = %d AND pg.game_id = %d", g_players[id][DB_ID], g_game_id);
	}
	//SQL_ThreadQuery(g_SqlTuple, "CommonQueryHandle", query)
	//server_print("query len = %d\n", strlen(query))
	//log_amx(query)
	//server_print(query)
	//server_print(query[254])
	//server_print(query[508])
	
	new Handle:query_handle = SQL_PrepareQuery(g_connection, query)
	/*
	if( !SQL_Execute(query_handle) )
	{
		new error[512]
		SQL_QueryError(query_handle, error, 511)
        	return set_fail_state(error)
	}*/
	
	if(check_execute(query_handle))
	{
		SQL_FreeHandle(query_handle)
		return 1
	}
	
	log_amx("Unable to save player name=%s, authid=%s, ip=%s, rank=%d, points=%d, kills=%d, deaths=%d, score=%d, time=%d",
		name, authid, ip,
		g_players[id][RANK],
		g_players[id][POINTS],
		g_players[id][KILLS] - g_players[id][ORIGIN_KILLS],
		g_players[id][DEATHS] - g_players[id][ORIGIN_DEATHS],
		g_players[id][SCORE] - g_players[id][ORIGIN_SCORE],
		get_user_time(id, 1))
	
	return 0
}

public check_loaded(task_id)
{
	new id = task_id - TASK_LOADED
	
	if( !is_user_connected(id) )
		return PLUGIN_CONTINUE
	
	if( !g_players[id][LOADED] )
		return hf_player_authenticated(id, g_players[id][DB_ID])
	
	return PLUGIN_CONTINUE
}

stock load_points_from_stats(id, db_player_id, ratio=0)
{
	if( ratio )
	{
		g_players[id][KILLS] = 0
		g_players[id][DEATHS] = 0
	}
	
	if( get_pcvar_num(g_cvar_load_rank) )
	{
		new stats[9], hits[8]
		
		if( get_user_wstats(id, 0, stats, hits) )
		{
			g_players[id][POINTS] += (stats[0] - stats[2]) * g_events[EVENT_KILL] // Kills
			g_players[id][POINTS] += stats[3] * g_events[EVENT_HS] // Headshots
			g_players[id][POINTS] += stats[7] * g_events[EVENT_SCORE] // Score
			g_players[id][POINTS] -= stats[1] * g_events[EVENT_DEATH] // Deaths
			
			if( g_players[id][POINTS] < 0 )
				g_players[id][POINTS] = 0
			
			if( ratio )
			{
				g_players[id][KILLS] = stats[0]
				g_players[id][DEATHS] = stats[1]
			}
		}
	}
	
	g_players[id][RANK] = rank_from_points( g_players[id][POINTS] )
	
	new query[300]
	formatex(query, 299,
		"INSERT INTO ranks_playergame (player_id, game_id, rank, points, kills, deaths, score, pratio, mratio, kpm, dpm, first_seen, last_activity, game_time) VALUES (%d, %d, %d, %d, 0, 0, 0, 0.0, 0.0, 0.0, 0.0, NOW(), NOW(), 0)",
		db_player_id, g_game_id, g_players[id][RANK], g_players[id][POINTS])
	//log_amx(query)
	new Handle:query_handle = SQL_PrepareQuery(g_connection, query)
	
	if(!check_execute(query_handle))
	{
		set_task(30.0, "check_loaded", TASK_LOADED + id)
		return 0
	}
	
	SQL_FreeHandle(query_handle)
	return 1;
}

stock degrade_player(id, player_rank=-1)
{
	if( g_players[id][RANK] > FIRST_RANK )
	{
		if( player_rank == -1 )
		{
			if( g_players[id][RANK] > FIRST_RANK )
				g_players[id][RANK] -= 1
		} else g_players[id][RANK] = player_rank
		
		if( g_players[id][RANK] < LAST_RANK && g_players[id][POINTS] > g_rankpoints[ g_players[id][RANK]+1 ] )
			g_players[id][POINTS] = g_rankpoints[ g_players[id][RANK] ]
		
		g_players[id][NOTIFIED] = g_rankpoints[LAST_RANK]
		
		new name[32], players[32], players_count, i
		
		get_players(players, players_count, "ch")
		get_user_name(id, name, 31)
		
		for(i = 0; i < players_count; i++)
			if( players[i] == id )
				client_print(players[i], print_chat, "%L", players[i], "SUSPENDED_YOU", players[i], RANKSTRINGS[ g_players[id][RANK] ])
			else
				client_print(players[i], print_chat, "%L", players[i], "SUSPENDED_PLAYER", name, players[i], RANKSTRINGS[ g_players[id][RANK] ])
	}
}

stock upgrade_player(id, player_rank=-1)
{
	if( g_players[id][RANK] < LAST_RANK )
	{
		if( player_rank == -1 )
		{
			if( g_players[id][RANK] < LAST_RANK )
				g_players[id][RANK] += 1
		} else g_players[id][RANK] = player_rank
		
		if( g_players[id][POINTS] < g_rankpoints[ g_players[id][RANK] ] )
			g_players[id][POINTS] = g_rankpoints[ g_players[id][RANK] ]
		
		g_players[id][NOTIFIED] = g_rankpoints[LAST_RANK]
		
		new name[32], players[32], players_count, i
		
		get_players(players, players_count, "ch")
		get_user_name(id, name, 31)
		
		for(i = 0; i < players_count; i++)
			if( players[i] == id )
				client_print(players[i], print_chat, "%L", players[i], "PROMOTED_YOU", players[i], RANKSTRINGS[ g_players[id][RANK] ])
			else
				client_print(players[i], print_chat, "%L", players[i], "PROMOTED_PLAYER", name, players[i], RANKSTRINGS[ g_players[id][RANK] ])
	}
}

stock points_to_next_level(id)
{
	if( g_players[id][RANK] < LAST_RANK )
		return g_rankpoints[ g_players[id][RANK]+1 ] - g_players[id][POINTS]
	return 0
}

stock points_add(id, num, verbose=1, ratio=1)
{
	if( ratio )
	{
		if( get_pcvar_num(g_cvar_apply_ratio) )
			num = floatround(num * pratio(id))
		if( get_pcvar_num(g_cvar_apply_kpm) )
			num = floatround(num * kpm(id))
	}
	
	g_players[id][POINTS] += floatround(num * g_server_weight)
	
	new new_rank = rank_from_points(g_players[id][POINTS])
	
	if( new_rank > g_players[id][RANK] )
		upgrade_player(id, new_rank)
	else if( verbose && g_players[id][RANK] < LAST_RANK ) {
		new remaining = points_to_next_level(id)
		if( remaining <= 20 || (remaining <= 50 && g_players[id][NOTIFIED] > 50) || (remaining <= 100 && g_players[id][NOTIFIED] > 100) || (remaining <= 200 && g_players[id][NOTIFIED] > 200))
		{
			g_players[id][NOTIFIED] = remaining
			client_print(id, print_chat, "%L", id, "POINTS_REMAINING", remaining, id, RANKSTRINGS[ g_players[id][RANK]+1 ])
		}
	}
	
#if defined SHOW_POINTS_HUD
	new str[6]
	formatex(str, 5, "+%d", num)
	show_hud(id, str)
#endif
}

stock points_remove(id, num, ratio=1, set_new_rank=0)
{
	if( ratio )
	{
		if( get_pcvar_num(g_cvar_apply_ratio) )
			num = floatround(num * mratio(id))
		if( get_pcvar_num(g_cvar_apply_kpm) )
			num = floatround(num * dpm(id))
	}
	
	g_players[id][POINTS] += floatround(num * g_server_weight)
	
	if( g_players[id][POINTS] < 0 )
		g_players[id][POINTS] = 0
	
	new new_rank = rank_from_points(g_players[id][POINTS])
	
#if defined RANK_CORRECTION
	if( new_rank < g_players[id][RANK] && g_players[id][POINTS] < (g_rankpoints[ g_players[id][RANK] ]/2) && g_players[id][RANK] > (FIRST_RANK+1) )
#else
	if( new_rank < g_players[id][RANK] && g_players[id][POINTS] < (g_rankpoints[ g_players[id][RANK] ]/2) )
#endif
		degrade_player(id, set_new_rank ? new_rank : -1)
	
#if defined SHOW_POINTS_HUD
	new str[6]
	formatex(str, 5, "-%d", num)
	show_hud(id, str)
#endif
}

stock rank_from_points(player_points)
{
	new i
	for(i = LAST_RANK; i >= FIRST_RANK; i--)
		if( player_points >= g_rankpoints[i] )
			return i
	return FIRST_RANK
}

stock reset_player(id)
{
	g_players[id][LOADED] = 0
	g_players[id][TYPE] = TYPE_IGNORE
	g_players[id][POINTS] = 0
	g_players[id][RANK] = 0
	g_players[id][KILLS] = 0
	g_players[id][DEATHS] = 0
	g_players[id][SCORE] = 0
	g_players[id][ORIGIN_KILLS] = 0
	g_players[id][ORIGIN_DEATHS] = 0
	g_players[id][ORIGIN_SCORE] = 0
	g_players[id][GAME_TIME] = 0
	g_players[id][PASSWORD] = 0
	g_players[id][DB_ID] = 0
}

stock check_execute(Handle:query)
{
	if( !SQL_Execute(query) )
	{
		new error[512]
		SQL_QueryError(query, error, 511)
		log_amx("SQL error: %s", error)
		
		if(equal(error, "MySQL server has gone away"))
		{
			g_loaded = 0
			SQL_FreeHandle(g_connection)
			g_connection = Empty_Handle
			
			if(!task_exists(TASK_CONFIGURE))
				set_task(15.0, "configure", TASK_CONFIGURE)
		}
		
		SQL_FreeHandle(query)
		
		return 0
	}
	
	return 1
}

#if defined SHOW_POINTS_HUD
stock show_hud(id, str[])
{
	set_hudmessage(
		255, // R
		100, // G
		100, // B
		0.95, // X
		0.35, // Y
		0, // effect
		0.0, // fxtime
		0.8, // holdtime
		0.1, // fadeintime
		0.15, // fadeouttime
		4 // channel
	)
	show_hudmessage(id, str)
}
#endif

stock Float:pratio(id)
{
	return ((g_players[id][DEATHS] == 0) ? float(g_players[id][KILLS]) : (float(g_players[id][KILLS]) / float(g_players[id][DEATHS])))
}

stock Float:mratio(id)
{
	return ((g_players[id][KILLS] == 0) ? float(g_players[id][DEATHS]) : (float(g_players[id][DEATHS]) / float(g_players[id][KILLS])))
}

#if defined RANK_CORRECTION
stock check_rank_correction(id)
{
	if(g_players[id][RANK] > FIRST_RANK)
		return PLUGIN_CONTINUE
	
	new len = get_pcvar_num(g_cvar_rank_correction_time) * 60
	new timeleft = get_timeleft()
	new remaining = len - g_players[id][GAME_TIME]
	
	if(remaining <= 0)
		upgrade_player(TASK_CORRECTION + id)
	else if(remaining < timeleft || timeleft == 0)
	{
		set_task(float(remaining), "promote_to_private", TASK_CORRECTION + id)
	}
	
	return PLUGIN_CONTINUE
}

public promote_to_private(task_id)
{
	new id = task_id - TASK_CORRECTION
	
	upgrade_player(id)
}
#endif

// id, &points, &remaining, &rank, &Float:pratio, &Float:mratio, rank_str[], len = RANK_MAX_LENGTH
public native_ranks_get_info()
{
	new id = get_param(1)
	new len = get_param(8)
	new str[RANK_MAX_LENGTH]
	
	formatex(str, len-1, "%L", id, RANKSTRINGS[ g_players[id][RANK] ])
	
	set_param_byref(2, g_players[id][POINTS])
	set_param_byref(3, points_to_next_level(id))
	set_param_byref(4, g_players[id][RANK])
	set_float_byref(5, pratio(id))
	set_float_byref(6, mratio(id))
	set_array(7, str, len-1)
	
	return 1
}
