/**
 * Copyright (C) 2010-2013 Jakub Skokan <aither@havefun.cz>
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3
 * as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <amxmodx>
#include <sqlx>
#include <hf_api>
#include <hf_utils>

#define VERSION "0.1.0"

enum _:Players {
	DB_ID,
	USER_ID,
	LOADED,
	TYPE,
	PASSWORD,
}

enum {
	TYPE_NONSTEAM=0,
	TYPE_STEAM,
	TYPE_IGNORE,
	
	TASK_CONFIGURE=40,
	TASK_LOADED=50,
	
	PLAYER_AUTHENTICATED=0,
	PLAYER_CHANGED,
	PLAYER_DISCONNECTED,
	FORWARDS_COUNT,
}

new g_players[33][Players]
new g_forwards[FORWARDS_COUNT]
new Handle:g_SqlTuple
new Handle:g_connection = Empty_Handle

new g_cvar_pass_field

new g_loaded = 0

public plugin_init()
{
	register_plugin("HF API", VERSION, "aither")
	
	g_cvar_pass_field = register_cvar("hf_password_field", "_hf_pass")
	
	// hf_player_authenticated(id, db_id)
	g_forwards[PLAYER_AUTHENTICATED] = CreateMultiForward("hf_player_authenticated", ET_IGNORE, FP_CELL, FP_CELL)
	
	// hf_player_changed(id, db_id)
	g_forwards[PLAYER_CHANGED] = CreateMultiForward("hf_player_changed", ET_IGNORE, FP_CELL, FP_CELL)
	
	// hf_player_disconnected(id, db_id)
	g_forwards[PLAYER_DISCONNECTED] = CreateMultiForward("hf_player_disconnected", ET_IGNORE, FP_CELL, FP_CELL)
}

public plugin_cfg()
{
	set_task(0.25, "configure", TASK_CONFIGURE)
}

public plugin_end()
{
	if(g_connection != Empty_Handle)
		SQL_FreeHandle(g_connection)
	
	SQL_FreeHandle(g_SqlTuple)
	
	new i
	for(i = 0; i < FORWARDS_COUNT; i++)
		DestroyForward(g_forwards[i])
}

public plugin_natives()
{
	register_native("hf_get_player_userid", "native_hf_get_player_userid")
}

public configure()
{
	if(g_connection == Empty_Handle)
	{
		new host[64], user[64], pass[64], db[64]
		get_cvar_string("amx_sql_host", host, 63)
		get_cvar_string("amx_sql_user", user, 63)
		get_cvar_string("amx_sql_pass", pass, 63)
		get_cvar_string("amx_sql_db", db, 63)
		
		g_SqlTuple = SQL_MakeDbTuple(host, user, pass, db)
		new error[512], errno
		g_connection = SQL_Connect(g_SqlTuple, errno, error, 511)
		
		if(g_connection == Empty_Handle)
		{
			// stop the plugin with an error message
			//set_fail_state(error)
			log_amx("Cannot connect to db, I'll try it again later.")
			set_task(30.0, "configure", TASK_CONFIGURE)
			
			return PLUGIN_CONTINUE
		}
	}
	
	g_loaded = 1
	
	return PLUGIN_CONTINUE
}

public client_putinserver(id)
{
	if( is_user_bot(id) || is_user_hltv(id) )
	{
		g_players[id][TYPE] = TYPE_IGNORE
		return PLUGIN_CONTINUE
	}
	
	return player_auth(id)
}

public client_infochanged(id)
{
	if( !is_user_connected(id) || is_user_bot(id) || is_user_hltv(id) )
	{
		g_players[id][TYPE] = TYPE_IGNORE
		return PLUGIN_CONTINUE
	}
	
	if( g_players[id][TYPE] == TYPE_NONSTEAM )
		return player_auth(id, 1)
	
	return PLUGIN_CONTINUE
}

public client_disconnect(id)
{
	if( g_players[id][TYPE] == TYPE_IGNORE || !g_players[id][LOADED] )
		return PLUGIN_CONTINUE
	
	new ret
	ExecuteForward(g_forwards[PLAYER_DISCONNECTED], ret, id, g_players[id][DB_ID])
	
	reset_player(id)
	
	return PLUGIN_CONTINUE
}

public check_loaded(task_id)
{
	new id = task_id - TASK_LOADED
	
	if( !is_user_connected(id) )
		return PLUGIN_CONTINUE
	
	if( !g_players[id][LOADED] )
		return player_auth(id)
	
	return PLUGIN_CONTINUE
}

public native_hf_get_player_userid()
{
	new id = get_param(1)
	return g_players[id][USER_ID]
}

stock player_auth(id, rename = 0)
{
	if( !is_user_connected(id) )
		return PLUGIN_CONTINUE
		
	if( !g_loaded )
	{
		set_task(10.0, "check_loaded", TASK_LOADED + id)
		return PLUGIN_CONTINUE
	}
	
	new name[32]
	
	if(rename)
	{
		new oldname[32] 
		get_user_info(id, "name", name,31) 
		get_user_name(id, oldname, 31)
		
		if( equal(name, oldname) || g_players[id][TYPE] == TYPE_STEAM )
			return PLUGIN_CONTINUE
		else {
			new ret
			ExecuteForward(g_forwards[PLAYER_CHANGED], ret, id, g_players[id][DB_ID])
		}
	} else get_user_name(id, name, 31)
	
	reset_player(id)
	
	mysql_escape_string(name, 31)
	new query[300], authid[33], ip[16]
	
	get_user_ip(id, ip, 15, 1)
	get_user_authid(id, authid, 32)
	
	if( strlen(ip) == 0 || strlen(authid) == 0 ) // Sometimes empty string is returned instead of IP or authid...
	{
		set_task(10.0, "check_loaded", TASK_LOADED + id)
		//log_amx("manage_player: Empty IP|authid on player %d - name=%s, authid=%s", id, name, authid)
		return PLUGIN_CONTINUE
	}
	
	mysql_escape_string(ip, 15)
	mysql_escape_string(authid, 32)
	
	if( equal(authid, "STEAM_ID_LAN") || equal(authid, "VALVE_ID_LAN") )
	{
		formatex(query, 299,
			"SELECT id, user_id, password FROM ranks_player WHERE name = '%s' AND (ip = '%s' OR password != '')",
			name, ip)
		g_players[id][TYPE] = TYPE_NONSTEAM
	} else {
		formatex(query, 299,
			"SELECT id, user_id, password FROM ranks_player WHERE authid = '%s'",
			authid)
		g_players[id][TYPE] = TYPE_STEAM
	}
	
	if(g_players[id][TYPE] == TYPE_IGNORE)
	{
		//log_amx("manage_player: Ignoring player %d - name=%s, ip=%s, authid=%s", id, name, ip, authid)
		return PLUGIN_CONTINUE
	}
	//log_amx(query)
	
	new Handle:query_handle = SQL_PrepareQuery(g_connection, query)
	
	if(!check_execute(query_handle))
	{
		set_task(30.0, "check_loaded", TASK_LOADED + id)
		return PLUGIN_CONTINUE
	}
	
	if( SQL_NumResults(query_handle) == 0 && g_players[id][TYPE] != TYPE_IGNORE ) // New client
	{
		formatex(query, 299, 
			"INSERT INTO ranks_player (name, authid, ip, kills, deaths, kpm, dpm, total_game_time, first_seen) VALUES ('%s', '%s', '%s', 0, 0, 0.0, 0.0, 0, NOW())",
			name, authid, ip)
		//log_amx(query)
		new Handle:query_newplayer = SQL_PrepareQuery(g_connection, query)
	
		if(!check_execute(query_newplayer))
		{
			set_task(30.0, "check_loaded", TASK_LOADED + id)
			return PLUGIN_CONTINUE
		}
		
		g_players[id][DB_ID] = SQL_GetInsertId(query_newplayer)
		
		SQL_FreeHandle(query_newplayer)
		SQL_FreeHandle(query_handle)
	} else {
		new pass[16], pass_field[9]
		get_pcvar_string(g_cvar_pass_field, pass_field, 8)
		get_user_info(id, pass_field, pass, 15)
		
		new db_pass[16]
		SQL_ReadResult(query_handle, SQL_FieldNameToNum(query_handle, "password"), db_pass, 15)
		
		if( strlen(db_pass) > 0)
		{
			if( !equal(pass, db_pass) )
			{
				g_players[id][TYPE] = TYPE_IGNORE
				server_cmd("kick #%d ^"%L^"", get_user_userid(id), id, "PROTECTED_NICK")
				log_amx("player_auth: Player kicked for bad password %d - name=%s, authid=%s, ip=%s", id, name, authid, ip)
				
				return PLUGIN_CONTINUE
			} else {
				g_players[id][PASSWORD] = 1
			}
		} else g_players[id][PASSWORD] = 0
			
		g_players[id][DB_ID] = SQL_ReadResult(query_handle, SQL_FieldNameToNum(query_handle, "id"))
		g_players[id][USER_ID] = SQL_ReadResult(query_handle, SQL_FieldNameToNum(query_handle, "user_id"))
	}
	
	g_players[id][LOADED] = 1
	
	new ret
	ExecuteForward( g_forwards[PLAYER_AUTHENTICATED], ret, id, g_players[id][DB_ID] )
	
	SQL_FreeHandle(query_handle)
	
	//log_amx("manage_player: Player loaded %d - name=%s, authid=%s, ip=%s, kills=%d, deaths=%d, points=%d, rank=%d", id, name, authid, ip, g_players[id][KILLS], g_players[id][DEATHS], g_players[id][POINTS], g_players[id][RANK])
	return PLUGIN_CONTINUE
}

stock reset_player(id)
{
	g_players[id][DB_ID] = 0
	g_players[id][LOADED] = 0
	g_players[id][TYPE] = TYPE_IGNORE
	g_players[id][PASSWORD] = 0
}

stock check_execute(Handle:query)
{
	if( !SQL_Execute(query) )
	{
		new error[512]
		SQL_QueryError(query, error, 511)
		log_amx("SQL error: %s", error)
		
		if(equal(error, "MySQL server has gone away"))
		{
			g_loaded = 0
			SQL_FreeHandle(g_connection)
			g_connection = Empty_Handle
			
			if(!task_exists(TASK_CONFIGURE))
				set_task(15.0, "configure", TASK_CONFIGURE)
		}
		
		SQL_FreeHandle(query)
		
		return 0
	}
	
	return 1
}
