/**
 * Copyright (C) 2010-2013 Jakub Skokan <aither@havefun.cz>
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3
 * as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <amxmodx>
#include <dodx>
#include <dodfun>
#include <sqlx>
#include <cellarray>
#include <fakemeta>
#include <sockets>
#include <time>
#include <hf_api>
#include <hf_dod_match>
#include <hf_utils>

#pragma dynamic 32768 

#define HF_TAG "DoD.HaveFun.cz"

enum _:Players {
	INDEX,
}

enum {
	TASK_CONFIGURE=40,
	TASK_DELAYED_CHANGE=50,
	TASK_LIVE=60,
	TASK_PRELIVE2=70,
	TASK_POST_LIVE_HF=75,
	TASK_POST_LIVE=80,
	TASK_UNPAUSE_COUNTDOWN=90,
	TASK_UNPAUSE=91,
	TASK_SHUTDOWN=100,
	TASK_READY_LOOP=110,
	TASK_RESULTS=120,
	TASK_TEAM_CHECK=130,
	TASK_HLTV_CHECK=140,
	TASK_HLTV_STOP_RECORDING=150,
	
	USER_ID=0,
	MATCH_USER_ID,
	TEAM_ID,
	
	S_KEEP_RUNNING=0,
	S_SHUTDOWN,
}

new Handle:g_SqlTuple
new Handle:g_connection = Empty_Handle
new g_loaded = 0
new g_reserved = 0

new g_internal_name[31]

new g_league_name[101]
new g_team_size = 0
new g_map_timelimit = 0
new g_map_half_timelimit = 0

new g_t1_id = 0
new g_t1_name[101]
new g_t1_points = 0
new g_t1_players = 0

new g_t2_id = 0
new g_t2_name[101]
new g_t2_points = 0
new g_t2_players = 0

new g_map[31]
new g_state = 0
new g_match_id = 0
new g_pause_count = 0
new g_pause_length = 0
new g_require_demos = 0
new g_hltv_present = 0
new g_hltv_ip[16], g_hltv_port
new g_hltv_recording = 0
new g_hltv_password[11], g_hltv_challenge[13]
new g_hltv_demo_prefix[101]
new bool:g_challenging_rcon

new g_players[33][Players]
new Array:g_valid_players = Array:0
new g_penalties[PENALTY_COUNT]

new g_t1_final_score, g_t2_final_score, g_winner
new g_t1_points_change, g_t2_points_change
new g_t1_round_team, g_t2_round_team

new g_cps[32]
new g_cp_count

new g_hudsync

new g_paused = 0
new g_pause_start = 0

new readystate = 0 // (when set to 1 we're in a readyrestart)
new axisready = 0
new alliesready = 0

// Cvars for match
new cvar_match, cvar_t1_score, cvar_t2_score
// Cvars for pure reservations
new cvar_score_allies, cvar_score_axis
new cvar_pause_t1, cvar_pause_t2
new cvar_pov_recording

public plugin_init()
{
	register_plugin("HF DoD Match", HF_VERSION, "aither")
	
	register_dictionary("hf_dod_match.txt")
	
	cvar_match = register_cvar("hf_match_state", "0")
	cvar_t1_score = register_cvar("hf_t1_score", "0")
	cvar_t2_score = register_cvar("hf_t2_score", "0")
	cvar_score_allies = register_cvar("hf_score_allies", "0")
	cvar_score_axis = register_cvar("hf_score_axis", "0")
	cvar_pause_t1 = register_cvar("hf_match_pause_t1", "0")
	cvar_pause_t2 = register_cvar("hf_match_pause_t2", "0")
	cvar_pov_recording = register_cvar("hf_pov_recording", "0")
	
	register_event("ClanTimer", "pre_live", "a") // Called when countdown starts
	register_event("30", "post_half", "a" ) // Called before map change, I guess
	register_event("HudText", "axis_ready", "bC", "1=#Clan_axis_ready")
	register_event("HudText", "allies_ready", "bC", "1=#Clan_allies_ready")
	
	register_clcmd("say ready", "cmd_ready")
	register_clcmd("say notready", "cmd_notready")
	//register_clcmd("jointeam", "handle_teamjoin")
	register_clcmd("say", "cmd_say")
	register_clcmd("say pause", "cmd_pause")
	register_clcmd("say continue", "cmd_continue")
	
	g_hudsync = CreateHudSyncObj()
}

public plugin_cfg()
{
	set_task(0.25, "configure", TASK_CONFIGURE)
}

public plugin_precache()
{
	register_forward(FM_KeyValue, "fm_keyvalue");
}

public plugin_end()
{
	if(task_exists(TASK_RESULTS))
		remove_task(TASK_RESULTS)
	
	if(g_connection != Empty_Handle)
		SQL_FreeHandle(g_connection)
	
	SQL_FreeHandle(g_SqlTuple)
}

public configure()
{
	/**
	 * We should learn what reservation are we in here, what map should be played
	 * and who can enter,
	 */
	
	if(g_connection == Empty_Handle)
	{
		new host[64], user[64], pass[64], db[64]
		get_cvar_string("amx_sql_host", host, 63)
		get_cvar_string("amx_sql_user", user, 63)
		get_cvar_string("amx_sql_pass", pass, 63)
		get_cvar_string("amx_sql_db", db, 63)
		
		g_SqlTuple = SQL_MakeDbTuple(host, user, pass, db)
		new error[512], errno
		g_connection = SQL_Connect(g_SqlTuple, errno, error, 511)
		
		if(g_connection == Empty_Handle)
		{
			// stop the plugin with an error message
			//set_fail_state(error)
			log_amx("Cannot connect to db, I'll try it again later.")
			set_task(30.0, "configure", TASK_CONFIGURE)
			
			return PLUGIN_CONTINUE
		}
	}
	
	new query_str[1920], ip[32], port[12]
	
	get_cvar_string("ip", ip, 15)
	get_cvar_string("port", port, 5)
	
	mysql_escape_string(ip, 31)
	mysql_escape_string(port, 11)
	
	// The only problem here is that scoring_enabled is not checked if it's null, because I'm unable to set it to null through web UI
	
	new query_parts[][] = {
		"SELECT l.id AS league_id, l.name AS league_name, l.team_size, l.map_duration, l.server_password, l.team_check, l.internal_name, l.hltv_demo_prefix,",
		"l.require_demos, l.pause_count, l.pause_length, hltv.bind_ip AS hltv_ip, hltv.port AS hltv_port, hltv.rcon_password AS hltv_password,",
		"t1.id AS t1_id, t1.name_ascii AS t1_name, t1.points AS t1_points,",
		"t2.id AS t2_id, t2.name_ascii AS t2_name, t2.points AS t2_points,",
		"map.name AS map, m.state, m.id AS match_id,",
		"IFNULL(lm.respawn_time, map.respawn_time) AS respawn_time,",
		"lm.scoring_enabled, IFNULL(lm.scoring_delay, map.scoring_delay) AS scoring_delay,",
		"IF(lm.axis_scoring_values = '' OR lm.axis_scoring_values IS NULL, map.axis_scoring_values, lm.axis_scoring_values) AS axis_scoring_values,",
		"IF(lm.allies_scoring_values = '' OR lm.allies_scoring_values IS NULL, map.allies_scoring_values, lm.allies_scoring_values) AS allies_scoring_values,",
		"IFNULL(lm.axis_scoring_bonus, map.axis_scoring_bonus) AS axis_scoring_bonus,",
		"IFNULL(lm.allies_scoring_bonus, map.allies_scoring_bonus) AS allies_scoring_bonus",
		"FROM",
		"((((((((league_reservation r LEFT JOIN league_match m ON r.id = m.reservation_id)",
		"INNER JOIN league_bookableserver bs ON r.server_id = bs.id)",
		"INNER JOIN halflife_server hs ON bs.server_id = hs.id)",
		"LEFT JOIN halflife_hltv hltv ON hltv.id = hs.hltv_id)",
		"INNER JOIN league_map map ON r.map_id = map.id)",
		"LEFT JOIN league_team t1 ON m.team_1_id = t1.id)",
		"LEFT JOIN league_team t2 ON m.team_2_id = t2.id)",
		"LEFT JOIN league_league l ON t1.league_id = l.id)",
		"LEFT JOIN league_leaguemap lm ON (lm.map_id = map.id AND lm.league_id = l.id)",
		"WHERE (m.state = %d OR m.state = %d OR m.state IS NULL) AND hs.bind_ip = '%s' AND hs.port = %s",
		"AND (CONVERT_TZ(NOW(), '%s', 'UTC') BETWEEN r.date AND DATE_ADD(r.date, INTERVAL l.reservation_time MINUTE))",
		"LIMIT 1"
	}
	
	new i, pos = 0, len = sizeof(query_parts)
	
	for(i = 0; i < len; i++)
	{
		pos += copy(query_str[pos], 1919-pos, query_parts[i])
		pos += copy(query_str[pos], 1919-pos, " ")
	}
	
	format(query_str, 1919, query_str, M_SETTLED, M_BEING_PLAYED, ip, port, HF_TZ)
	new Handle:query = SQL_PrepareQuery(g_connection, query_str)
	
	if(!check_execute(query))
		return PLUGIN_CONTINUE
	
	if(!SQL_NumResults(query)) // No reservation
	{
		log_amx("No reservation found")
		
		SQL_FreeHandle(query)
		
		formatex(query_str, 1791, "SELECT no_reservation FROM halflife_server hs INNER JOIN league_bookableserver bs ON bs.server_id = hs.id WHERE hs.bind_ip = '%s' AND hs.port = %s",
			ip, port
		)
		
		query = SQL_PrepareQuery(g_connection, query_str)
		
		if(!check_execute(query) || !SQL_NumResults(query))
			return PLUGIN_CONTINUE
		
		switch( SQL_ReadResult(query, SQL_FieldNameToNum(query, "no_reservation")) )
		{
			case S_KEEP_RUNNING: {
				return PLUGIN_CONTINUE
			}
			case S_SHUTDOWN: {
				SQL_FreeHandle(query)
				shutdown()
				return PLUGIN_CONTINUE
			}
		}
	} else {
		// First get reservation data
		SQL_ReadResult(query, SQL_FieldNameToNum(query, "map"), g_map, 30)
		
		new current_map[32]
		get_mapname(current_map, 31)
		
		if(!equali(current_map, g_map))
		{
			log_amx("Changing map to reserved one")
			server_cmd("changelevel %s", g_map)
			return PLUGIN_CONTINUE
		}
		
		new values_allies[10], values_axis[10]
		
		SQL_ReadResult(query, SQL_FieldNameToNum(query, "allies_scoring_values"), values_allies, 9)
		SQL_ReadResult(query, SQL_FieldNameToNum(query, "axis_scoring_values"), values_axis, 9)
		
		fix_flags_order(values_allies, 9)
		fix_flags_order(values_axis, 9)
		
		set_cvar_num("mp_clan_respawntime", SQL_ReadResult(query, SQL_FieldNameToNum(query, "respawn_time")))
		set_cvar_num("mp_clan_scoring", SQL_ReadResult(query, SQL_FieldNameToNum(query, "scoring_enabled")))
		set_cvar_num("mp_clan_scoring_delay", SQL_ReadResult(query, SQL_FieldNameToNum(query, "scoring_delay")))
		set_cvar_string("mp_clan_scoring_values_axis", values_axis)
		set_cvar_string("mp_clan_scoring_values_allies", values_allies)
		set_cvar_num("mp_clan_scoring_bonus_axis", SQL_ReadResult(query, SQL_FieldNameToNum(query, "axis_scoring_bonus")))
		set_cvar_num("mp_clan_scoring_bonus_allies", SQL_ReadResult(query, SQL_FieldNameToNum(query, "allies_scoring_bonus")))
		
		// Now match info, if some is supposed to be played
		g_match_id = SQL_ReadResult(query, SQL_FieldNameToNum(query, "match_id"))
		
		if(!g_match_id) // No match here, just reservation
		{
			log_amx("Found reservation: map: %s, reserved by: %d", g_map, 0)
		} else {
			new col_league_name = SQL_FieldNameToNum(query, "league_name")
			new col_team_size = SQL_FieldNameToNum(query, "team_size")
			new col_map_duration = SQL_FieldNameToNum(query, "map_duration")
			new col_t1_id = SQL_FieldNameToNum(query, "t1_id")
			new col_t1_name = SQL_FieldNameToNum(query, "t1_name")
			new col_t2_id = SQL_FieldNameToNum(query, "t2_id")
			new col_t2_name = SQL_FieldNameToNum(query, "t2_name")
			new col_state = SQL_FieldNameToNum(query, "state")
			
			new sv_password[11]
			
			SQL_ReadResult(query, col_league_name, g_league_name, 100)
			g_team_size = SQL_ReadResult(query, col_team_size)
			g_map_timelimit = SQL_ReadResult(query, col_map_duration)
			
			SQL_ReadResult(query, SQL_FieldNameToNum(query, "server_password"), sv_password, 10)
			
			g_pause_count = SQL_ReadResult(query, SQL_FieldNameToNum(query, "pause_count"))
			g_pause_length = SQL_ReadResult(query, SQL_FieldNameToNum(query, "pause_length"))
			
			new col_hltv_ip = SQL_FieldNameToNum(query, "hltv_ip")
			
			if(!SQL_IsNull(query, col_hltv_ip))
			{
				SQL_ReadResult(query, col_hltv_ip, g_hltv_ip, 15)
				g_hltv_port = SQL_ReadResult(query, SQL_FieldNameToNum(query, "hltv_port"))
				SQL_ReadResult(query, SQL_FieldNameToNum(query, "hltv_password"), g_hltv_password, 10)
			}
			
			g_require_demos = SQL_ReadResult(query, SQL_FieldNameToNum(query, "require_demos"))
			
			SQL_ReadResult(query, SQL_FieldNameToNum(query, "hltv_demo_prefix"), g_hltv_demo_prefix, 100)
			
			SQL_ReadResult(query, SQL_FieldNameToNum(query, "internal_name"), g_internal_name, 30)
			
			g_t1_id = SQL_ReadResult(query, col_t1_id)
			SQL_ReadResult(query, col_t1_name, g_t1_name, 100)
			g_t1_points = SQL_ReadResult(query, SQL_FieldNameToNum(query, "t1_points"))
			
			g_t2_id = SQL_ReadResult(query, col_t2_id)
			SQL_ReadResult(query, col_t2_name, g_t2_name, 100)
			g_t2_points = SQL_ReadResult(query, SQL_FieldNameToNum(query, "t2_points"))
			
			g_state = SQL_ReadResult(query, col_state)
			
			g_map_half_timelimit = g_map_timelimit / 2
			
			new team_check = SQL_ReadResult(query, SQL_FieldNameToNum(query, "team_check")) * 60
			
			if(team_check)
				set_task(float(team_check), "team_check", TASK_TEAM_CHECK)
			
			
			log_amx("Found match: League %s (%don%d) - %s vs %s on %s", g_league_name, g_team_size, g_team_size, g_t1_name, g_t2_name, g_map)
			
			g_reserved = 1
			
			set_cvar_string("sv_password", sv_password)
			
			readystate = 1
			
			formatex(query_str, 1023, "SELECT ut.user_id, ut.team_id FROM (league_userteam ut INNER JOIN league_team t ON t.id = ut.team_id) INNER JOIN league_league l ON l.id = t.league_id WHERE (team_id = %d OR team_id = %d) AND state = 1 AND (DATE_ADD(l.start, INTERVAL 14 DAY) >= CONVERT_TZ(NOW(), '%s', 'UTC') OR DATE_ADD(joined_date, INTERVAL 1 DAY) < CONVERT_TZ(NOW(), '%s', 'UTC'))",
				g_t1_id, g_t2_id, HF_TZ, HF_TZ
			)
			
			new Handle:query_players = SQL_PrepareQuery(g_connection, query_str)
			
			if(!check_execute(query_players))
				return PLUGIN_CONTINUE
			
			if(g_valid_players != Array:0)
				ArrayClear(g_valid_players)
			else
				g_valid_players = ArrayCreate(3, g_team_size * 2 + 1)
			
			if(!SQL_NumResults(query_players))
			{
				// What to do?
				server_print("*** No valid players found")
			} else {
				while(SQL_MoreResults(query_players))
				{
					new player[3]
					player[USER_ID] = SQL_ReadResult(query_players, SQL_FieldNameToNum(query_players, "user_id"))
					player[MATCH_USER_ID] = 0
					player[TEAM_ID] = SQL_ReadResult(query_players, SQL_FieldNameToNum(query_players, "team_id"))
					
					log_amx("Found valid player user_id=%d, team_id=%d", player[USER_ID], player[TEAM_ID])
					
					ArrayPushArray(g_valid_players, player)
					
					SQL_NextRow(query_players)
				}
			}
			
			SQL_FreeHandle(query_players)
			
			formatex(query_str, 1023, "SELECT event, points FROM league_leaguepenalty WHERE league_id = %d", SQL_ReadResult(query, SQL_FieldNameToNum(query, "league_id")))
			
			new Handle:query_penalties = SQL_PrepareQuery(g_connection, query_str)
			
			if(!check_execute(query_penalties))
				return PLUGIN_CONTINUE
			
			if(SQL_NumResults(query_penalties))
			{
				new event
				
				while(SQL_MoreResults(query_penalties))
				{
					event = SQL_ReadResult(query_penalties, SQL_FieldNameToNum(query_penalties, "event"))
					
					if(event > PENALTY_COUNT)
					{
						log_amx("Compatibility error: unsupported event %d", event)
						continue
					}
					
					g_penalties[event] = SQL_ReadResult(query_penalties, SQL_FieldNameToNum(query_penalties, "points"))
					
					SQL_NextRow(query_penalties)
				}
			}
			
			SQL_FreeHandle(query_penalties)
			
			set_task(30.0, "ready_loop", TASK_READY_LOOP)
		}
		
		set_cvar_num("mp_clan_match", 1)
		set_cvar_num("mp_clan_readyrestart", 1)
		set_cvar_num("mp_timelimit", 0)
	}
	
	SQL_FreeHandle(query)
	
	if(g_hltv_port)
		set_task(20.0, "hltv_check", TASK_HLTV_CHECK)
	
	g_loaded = 1
	
	return PLUGIN_CONTINUE
}

public client_putinserver(id)
{
	if(is_user_hltv(id))
	{
		new hltv_ipport[32], hltv_ip[16], hltv_port = 0
		get_user_ip(id, hltv_ipport, 31)
		
		strtok(hltv_ipport, hltv_ip, 15, hltv_ipport, 5, ':')
		hltv_port = str_to_num(hltv_ipport)
		
		if(!equal(hltv_ip, g_hltv_ip) || hltv_port != g_hltv_port)
		{
			server_cmd("kick #%d", get_user_userid(id))
			return PLUGIN_CONTINUE
		}
		
		// Rather stop recording...
		//if(g_reserved && get_pcvar_num(cvar_match))
		//{
		hltv_rcon_command("stoprecording")
		g_hltv_recording = 0
		//}
		
		g_hltv_present = 1
	}
	
	return PLUGIN_CONTINUE
}

public client_disconnect(id)
{
	if(is_user_hltv(id))
	{
		new hltv_ipport[32], hltv_ip[16], hltv_port = 0
		get_user_ip(id, hltv_ipport, 31)
		
		strtok(hltv_ipport, hltv_ip, 15, hltv_ipport, 5, ':')
		hltv_port = str_to_num(hltv_ipport)
		
		if(!equal(hltv_ip, g_hltv_ip) || hltv_port != g_hltv_port)
			return PLUGIN_CONTINUE
		
		g_hltv_recording = 0
		g_hltv_present = 0
	}
	
	return PLUGIN_CONTINUE
}

public hf_player_authenticated(id, db_id)
{
	/**
	 * Here we'll check if player is allowed to be here.
	 * He has to be in one of the two teams who have match.
	 */
	
	if(!g_reserved)
	{
		return PLUGIN_CONTINUE
	} else if(!g_match_id)
		return PLUGIN_CONTINUE
	
	new cnt = ArraySize(g_valid_players), i
	new userid = hf_get_player_userid(id), player[3]
	
	for(i = 0; i < cnt; i++)
	{
		ArrayGetArray(g_valid_players, i, player)
		
		if(userid == player[USER_ID])
		{
			if(player[TEAM_ID] == g_t1_id)
			{
				g_t1_players++
				
				if(g_t1_players == g_team_size)
				{
					client_print(0, print_chat, "%L", LANG_PLAYER, "TEAM_FULL", HF_TAG, g_t1_name)
				}
			} else {
				g_t2_players++
				
				if(g_t2_players == g_team_size)
				{
					client_print(0, print_chat, "%L", LANG_PLAYER, "TEAM_FULL", HF_TAG, g_t2_name)
				}
			}
			
			if(g_t1_players > g_team_size || g_t2_players > g_team_size)
			{
				server_cmd("kick #%d %L", get_user_userid(id), id, "KICK_FULL_TEAM")
				return PLUGIN_CONTINUE
			}
			
			//server_print("Valid player connected playerid=%d, userid=%d", db_id, player[USER_ID])
			g_players[id][INDEX] = i
			
			if(!player[MATCH_USER_ID])
			{
				new query_str[256]
				formatex(query_str, 255, "INSERT INTO league_matchuser (match_id, user_id, team_id) VALUES (%d, %d, %d) ON DUPLICATE KEY UPDATE id = LAST_INSERT_ID(id)", // Note that it's leagueteam id, not team id
					g_match_id, player[USER_ID], (player[TEAM_ID] == g_t1_id) ? g_t1_id : g_t2_id
				)
				
				new Handle:query = SQL_PrepareQuery(g_connection, query_str)

				if(check_execute(query))
				{
					player[MATCH_USER_ID] = SQL_GetInsertId(query)
					ArraySetArray(g_valid_players, i, player)
					
					SQL_FreeHandle(query)
				}
			}
			
			if(player[MATCH_USER_ID])
				add_player_history(id, PL_JOINED)
			
			return PLUGIN_CONTINUE
		}
	}
	
	server_cmd("kick #%d %L", get_user_userid(id), id, "KICK_NOT_MEMBER")
	//server_print("Player kicked")
	
	return PLUGIN_HANDLED
}

public hf_player_disconnected(id, db_id)
{
	/**
	 * If player leaves during match, he has 5 minutes to return,
	 * then his team must play without him.
	 */
	
	if(!g_reserved)
	{
		return PLUGIN_CONTINUE
	} else if(!g_match_id)
		return PLUGIN_CONTINUE
	
	new player[3]
	
	ArrayGetArray(g_valid_players, g_players[id][INDEX], player)
	
	if(player[TEAM_ID] == g_t1_id)
		g_t1_players--
	else g_t2_players--
	
	add_player_history(id, PL_LEFT)
	
	return PLUGIN_CONTINUE
}

public axis_ready(id)
{
	axisready = 1
	client_print(0, print_chat, "%L", LANG_PLAYER, "READY_AXIS", HF_TAG)
	return PLUGIN_CONTINUE
}

public allies_ready(id)
{
	alliesready = 1
	client_print(0, print_chat, "%L", LANG_PLAYER, "READY_ALLIES", HF_TAG)
	return PLUGIN_CONTINUE
}

public cmd_ready(id)
{
	if(readystate)
	{
		new team = get_user_team(id)
		
		if( (team == ALLIES && !alliesready) || (team == AXIS && !axisready) )
			add_player_history(id, PL_READY)
	}
	
	return PLUGIN_CONTINUE
}

public cmd_notready(id)
{
	new authid[32], name[32], teamname[16], teamid
	
	get_user_authid(id,authid,31)
	get_user_name(id,name,31)
	get_user_team(id,teamname,16)
	teamid = get_user_team(id)
	
	log_amx("^"%s<%d><%s><%s>^" triggered ^"notready^"", name, get_user_userid(id), authid, teamname)
	
	if(readystate == 1 && teamid == ALLIES && alliesready == 1)
	{
		alliesready = 0
		set_cvar_num("mp_clan_readyrestart", 1)
		client_print(0, print_chat, "%L", LANG_PLAYER, "READY_ALLIES_REVOKE", HF_TAG)
		add_player_history(id, PL_NOTREADY)
		
		return PLUGIN_CONTINUE
	} else if(readystate == 1 && teamid == AXIS && axisready == 1) {
		axisready = 0
		set_cvar_num("mp_clan_readyrestart", 1)
		client_print(0, print_chat, "%L", LANG_PLAYER, "READY_AXIS_REVOKE", HF_TAG)
		add_player_history(id, PL_NOTREADY)
		
		return PLUGIN_CONTINUE
	}

	return PLUGIN_CONTINUE
}

// public handle_teamjoin(id)
// {
// 	if(is_user_bot(id) || is_user_hltv(id))
// 		return PLUGIN_CONTINUE
// 	
// 	new team_s[2]
// 	read_argv(1,team_s,2)
// 	new team = str_to_num(team_s)
// }

public cmd_say(id)
{
	if(g_paused)
	{
		new name[32], msg[256]
		
		get_user_name(id, name, 31)
		read_args(msg, 255)
		remove_quotes(msg)
		
		client_print(0, print_chat, "%s: %s", name, msg)
		
		if(equali(msg, "remaining"))
		{
			new remaining = (g_pause_start + g_pause_length) - get_systime()
			
			if(remaining <= 0)
				client_print(0, print_chat, "%L", LANG_PLAYER, "BREAK_OVER", HF_TAG)
			else {
				new players[32], cnt, remaining_str[41]
				
				get_players(players, cnt, "ch")
				
				for(new i = 0; i < cnt; i++)
				{
					get_time_length(players[i], remaining, timeunit_seconds, remaining_str, 40)
					client_print(players[i], print_chat, "%L", players[i], "BREAK_REMAINING", HF_TAG, remaining_str)
				}
			}
		}
	}
	
	return PLUGIN_CONTINUE
}

public cmd_pause(id)
{
	if( !g_reserved || !get_pcvar_num(cvar_match) || g_paused )
		return PLUGIN_CONTINUE
	
	new player[3]
	ArrayGetArray(g_valid_players, g_players[id][INDEX], player)
	
	if(player[TEAM_ID] == g_t1_id)
	{
		if(get_pcvar_num(cvar_pause_t1) < g_pause_count)
		{
			set_pcvar_num(cvar_pause_t1, get_pcvar_num(cvar_pause_t1) + 1)
			g_paused = g_t1_id
		}
	} else {
		if(get_pcvar_num(cvar_pause_t2) < g_pause_count)
		{
			set_pcvar_num(cvar_pause_t2, get_pcvar_num(cvar_pause_t2) + 1)
			g_paused = g_t2_id
		}
	}
	
	if(!g_paused)
	{
		client_print(id, print_chat, "%L", id, "BREAK_USED", HF_TAG)
		return PLUGIN_HANDLED
	}
	
	add_player_history(id, PL_PAUSED)
	
	new players[32], cnt, break_length[41]
	
	get_players(players, cnt, "ch")
	
	for(new i = 0; i < cnt; i++)
	{
		get_time_length(players[i], g_pause_length, timeunit_seconds, break_length, 40)
		client_print(players[i], print_chat, "%L", players[i], "BREAK_REQUEST", HF_TAG, (g_paused == g_t1_id) ? g_t1_name : g_t2_name, break_length)
	}
	
	g_pause_start = get_systime()
	server_cmd("amx_pause")
	
	return PLUGIN_CONTINUE
}

public cmd_continue(id)
{
	if( !g_reserved || !get_pcvar_num(cvar_match) || !g_paused )
		return PLUGIN_CONTINUE
		
	new player[3]
	ArrayGetArray(g_valid_players, g_players[id][INDEX], player)
	
	if(player[TEAM_ID] == g_paused)
	{
		g_paused = 0
	} else if( (g_pause_start + g_pause_length) < get_systime() ) {
		client_print(0, print_chat, "%L", LANG_PLAYER, "BREAK_OVER_CONTINUE", HF_TAG)
		g_paused = 0
	} else {
		client_print(id, print_chat, "%L", id, "BREAK_WAIT", HF_TAG)
	}
	
	if(!g_paused)
	{
		add_player_history(id, PL_UNPAUSED)
		server_cmd("amx_pause")
	}
	
	return PLUGIN_CONTINUE
}
// 
// public unpause_countdown()
// {
// 	client_print(0, print_chat, "The game will continue in 10 seconds")
// 	set_task(10.0, "unpause_game", TASK_UNPAUSE)
// }
// 
// public unpause_game()
// {
// 	client_print(0, print_chat, "Good luck!")
// 	set_pev( 1, pev_flags, pev( 1, pev_flags ) & ~FL_FROZEN ) 
// 	g_paused = 0
// }

public team_check()
{
	new players[32], players_count, player[3], t1 = 0, t2 = 0
	
	get_players(players, players_count, "ch")
	
	for(new i = 0; i < players_count; i++)
	{
		ArrayGetArray(g_valid_players, g_players[players[i]][INDEX], player)
		
		if(player[TEAM_ID] == g_t1_id)
			t1 = 1
		else if(player[TEAM_ID] == g_t2_id)
			t2 = 1
		
		if(t1 && t2)
			return
	}
	
	// One or both teams is missing => end match + add penalties
	new query_str[256]
	formatex(query_str, 255, "UPDATE league_match SET state = %d WHERE id = %d LIMIT 1", M_NOT_STARTED, g_match_id)
	
	if(!SQL_SimpleQuery(g_connection, query_str))
	{
		log_amx("Unable to set match as not started")
	}
	
	if(!t1)
	{
		add_penalty(g_t1_id, PENALTY_NOT_PRESENT)
		client_print(0, print_chat, "%L", LANG_PLAYER, "TEAM_NOT_PRESENT", HF_TAG, g_t1_name, g_penalties[PENALTY_NOT_PRESENT])
		log_amx("%L", LANG_SERVER, "TEAM_NOT_PRESENT", HF_TAG, g_t1_name, g_penalties[PENALTY_NOT_PRESENT])
	}
	if(!t2)
	{
		add_penalty(g_t2_id, PENALTY_NOT_PRESENT)
		client_print(0, print_chat, "%L", LANG_PLAYER, "TEAM_NOT_PRESENT", HF_TAG, g_t2_name, g_penalties[PENALTY_NOT_PRESENT])
		log_amx("%L", LANG_SERVER, "TEAM_NOT_PRESENT", HF_TAG, g_t2_name, g_penalties[PENALTY_NOT_PRESENT])
	}
	
	client_print(0, print_chat, "%L", LANG_PLAYER, "SERVER_SHUTDOWN", HF_TAG, 20)
	
	set_task(20.0, "shutdown", TASK_SHUTDOWN)
}

public post_half()
{
	if(get_cvar_num("mp_clan_match") != 1)
		return
	
	if(get_pcvar_num(cvar_match) == 1) // First half
	{
		set_cvar_num("mp_chattime", 30)
		
		new authid[32], name[32]
		get_user_authid(0, authid, 31)
		get_user_name(0, name, 31)
		
		if(!g_match_id)
		{
			log_amx("^"%s<%d><%s><%s>^" say ^"End of round scores: Allies: %d   Axis: %d^"",
				name, get_user_userid(0), authid, HF_TAG, dod_get_team_score(ALLIES),
				dod_get_team_score(AXIS)
			)
		
			client_print(0, print_chat, "%L", LANG_PLAYER, "EOR_SCORES",
				HF_TAG, "Allies", dod_get_team_score(ALLIES), "Axis", dod_get_team_score(AXIS)
			)
			
			set_pcvar_num(cvar_score_allies, dod_get_team_score(ALLIES))
			set_pcvar_num(cvar_score_axis, dod_get_team_score(AXIS))
		} else {
			new t1_current_team = get_game_team(g_t1_id), t2_current_team = get_game_team(g_t2_id)
			
			if(t1_current_team == -1) // Not present
			{
				add_penalty(g_t1_id, PENALTY_NOT_PRESENT)
				client_print(0, print_chat, "%L", LANG_PLAYER, "TEAM_NOT_PRESENT", HF_TAG, g_t1_name, g_penalties[PENALTY_NOT_PRESENT])
				log_amx("%L", LANG_SERVER, "TEAM_NOT_PRESENT", HF_TAG, g_t1_name, g_penalties[PENALTY_NOT_PRESENT])
				
				if(t2_current_team != -1)
				{
					t1_current_team = t2_current_team == ALLIES ? AXIS : ALLIES
				} else {
					if(g_t1_round_team != -1)
						t1_current_team = g_t1_round_team
					else if(g_t2_round_team != -1)
						t1_current_team = g_t2_round_team == ALLIES ? AXIS : ALLIES
					else {
						log_amx("Cannot decide who's on which side, score allies=%d, axis=%d'", dod_get_team_score(ALLIES), dod_get_team_score(AXIS))
						return
					}
				}
			}
			
			if(t2_current_team == -1 ) // Not present neither
			{
				add_penalty(g_t2_id, PENALTY_NOT_PRESENT)
				client_print(0, print_chat, "%L", LANG_PLAYER, "TEAM_NOT_PRESENT", HF_TAG, g_t2_name, g_penalties[PENALTY_NOT_PRESENT])
				log_amx("%L", LANG_SERVER, "TEAM_NOT_PRESENT", HF_TAG, g_t2_name, g_penalties[PENALTY_NOT_PRESENT])
			}
			
			if(t1_current_team == ALLIES)
			{
				set_pcvar_num(cvar_t1_score, dod_get_team_score(ALLIES))
				set_pcvar_num(cvar_t2_score, dod_get_team_score(AXIS))
			} else {
				set_pcvar_num(cvar_t1_score, dod_get_team_score(AXIS))
				set_pcvar_num(cvar_t2_score, dod_get_team_score(ALLIES))
			}
			
			log_amx("^"%s<%d><%s><%s>^" say ^"End of round scores: %s: %d   %s: %d^"",
				name, get_user_userid(0), authid, HF_TAG, g_t1_name, get_pcvar_num(cvar_t1_score),
				g_t2_name, get_pcvar_num(cvar_t2_score)
			)
			
			client_print(0, print_chat, "%L", LANG_PLAYER, "EOR_SCORES",
				HF_TAG, g_t1_name, get_pcvar_num(cvar_t1_score), g_t2_name, get_pcvar_num(cvar_t2_score)
			)
			client_print(0, print_chat, "%L", LANG_PLAYER, "EOR_SCREENS", HF_TAG)
			client_cmd(0, "wait;wait;snapshot")
		}
		
		set_task(15.0, "delayed_change", TASK_DELAYED_CHANGE)
	} else if(get_pcvar_num(cvar_match) == 2) { // Final half
		set_cvar_num("mp_chattime", 60)
		
		new authid[32], name[32]
		get_user_authid(0,authid,31)
		get_user_name(0,name,31)
		
		if(!g_match_id)
		{
			log_amx("^"%s<%d><%s><%s>^" say ^"End of round scores: Allies: %d   Axis: %d^"",
				name, get_user_userid(0), authid, HF_TAG, dod_get_team_score(ALLIES),
				dod_get_team_score(AXIS)
			)
			
			client_print(0, print_chat, "%L", LANG_PLAYER, "EOR_SCORES", HF_TAG,
				"Allies", dod_get_team_score(ALLIES), "Axis", dod_get_team_score(AXIS)
			)
			
			g_t1_final_score = dod_get_team_score(AXIS) + get_pcvar_num(cvar_score_allies)
			g_t2_final_score = dod_get_team_score(ALLIES) + get_pcvar_num(cvar_score_axis)
			
			if(g_t1_final_score > g_t2_final_score)
				g_winner = ALLIES
			else if(g_t1_final_score < g_t2_final_score)
				g_winner = AXIS
			else
				g_winner = 0
			
			print_results()
			set_task(5.0, "print_results", TASK_RESULTS, "", 0, "b")
			
			set_task(20.0, "delayed_change", TASK_DELAYED_CHANGE)
			
			cleanup()
			
			return
		}
		
		
		new t1_score_allies, t1_score_axis, t2_score_allies, t2_score_axis
		new t1_round_score, t2_round_score
		new t1_current_team = get_game_team(g_t1_id), t2_current_team = get_game_team(g_t2_id)
			
		if(t1_current_team == -1) // Not present
		{
			add_penalty(g_t1_id, PENALTY_NOT_PRESENT)
			client_print(0, print_chat, "%L", LANG_PLAYER, "TEAM_NOT_PRESENT", HF_TAG, g_t1_name, g_penalties[PENALTY_NOT_PRESENT])
			log_amx("%L", LANG_SERVER, "TEAM_NOT_PRESENT", HF_TAG, g_t1_name, g_penalties[PENALTY_NOT_PRESENT])
			
			if(t2_current_team != -1)
			{
				t1_current_team = t2_current_team == ALLIES ? AXIS : ALLIES
			} else {
				if(g_t1_round_team != -1)
					t1_current_team = g_t1_round_team
				else if(g_t2_round_team != -1)
					t1_current_team = g_t2_round_team == ALLIES ? AXIS : ALLIES
				else {
					log_amx("Cannot decide who's on which side, score allies=%d, axis=%d'", dod_get_team_score(ALLIES), dod_get_team_score(AXIS))
					return
				}
			}
		}
		
		if(t2_current_team == -1 ) // Not present neither
		{
			add_penalty(g_t2_id, PENALTY_NOT_PRESENT)
			client_print(0, print_chat, "%L", LANG_PLAYER, "TEAM_NOT_PRESENT", HF_TAG, g_t2_name, g_penalties[PENALTY_NOT_PRESENT])
			log_amx("%L", LANG_SERVER, "TEAM_NOT_PRESENT", HF_TAG, g_t2_name, g_penalties[PENALTY_NOT_PRESENT])
		}
		
		if(t1_current_team == ALLIES)
		{
			t1_round_score = dod_get_team_score(ALLIES)
			t2_round_score = dod_get_team_score(AXIS)
		} else {
			t1_round_score = dod_get_team_score(AXIS)
			t2_round_score = dod_get_team_score(ALLIES)
		}
		
		g_t1_final_score = get_pcvar_num(cvar_t1_score) + t1_round_score
		g_t2_final_score = get_pcvar_num(cvar_t2_score) + t2_round_score
		
		if(t1_current_team == ALLIES)
		{
			t1_score_allies = t1_round_score
			t1_score_axis = g_t1_final_score - t1_round_score
			t2_score_allies = g_t2_final_score - t2_round_score
			t2_score_axis = t2_round_score
		} else {
			t1_score_allies = g_t1_final_score - t1_round_score
			t1_score_axis = t1_round_score
			t2_score_allies = t2_round_score
			t2_score_axis = g_t2_final_score - t2_round_score
		}
		
		if(g_t1_final_score > g_t2_final_score)
			g_winner = g_t1_id
		else if(g_t1_final_score < g_t2_final_score)
			g_winner = g_t2_id
		else
			g_winner = 0

		log_amx("^"%s<%d><%s><%s>^" say ^"End of round scores: %s: %d   %s: %d^"",
			name, get_user_userid(0), authid, HF_TAG,
			g_t1_name, t1_round_score, g_t2_name, t2_round_score
		)
		
		client_print(0, print_chat, "%L", LANG_PLAYER, "EOR_SCORES", HF_TAG,
			     g_t1_name, t1_round_score, g_t2_name, t2_round_score
		)

// 		log_amx("^"%s<%d><%s><%s>^" say ^"Final score: %s: %d   %s: %d^"", name, get_user_userid(0), authid, HF_TAG,
// 			g_t1_name, g_t1_final_score, g_t2_name, g_t2_final_score
//        		)
		
		calc_elo(g_t1_points, g_t2_points, g_t1_final_score, g_t2_final_score)
		
		print_results()
		set_task(5.0, "print_results", TASK_RESULTS, "", 0, "b")
		
		client_print(0, print_chat, "%L", LANG_PLAYER, "EOR_SCREENS", HF_TAG)
		client_cmd(0, "wait;wait;snapshot")
		
		log_amx("Match id=%d ended", g_match_id)
		log_amx("Team 1 final score: %d, score allies: %d, score axis: %d, points: %d",
			g_t1_final_score, t1_score_allies, t1_score_axis, g_t1_points_change
		)
		log_amx("Team 2 final score: %d, score allies: %d, score axis: %d, points: %d",
			g_t2_final_score, t2_score_allies, t2_score_axis, g_t2_points_change
		)
		log_amx("Match won: %d", g_winner)
		
		if(get_pcvar_num(cvar_pov_recording))
		{
			new players[32], count
			
			get_players(players, count, "ch")
			
			for(new i = 0; i < count; i++)
				client_cmd(players[i], "stop")
		}
		
		if(g_hltv_recording)
		{
			set_task(10.0, "hltv_stop_recording", TASK_HLTV_STOP_RECORDING)
		}
		
		new query_str[512]
		new query_parts[][] = {
			"UPDATE league_match SET state = %d,",
			"team_1_score = %d, team_1_score_allies = %d, team_1_score_axis = %d, team_1_points = team_1_points + %d,",
			"team_2_score = %d, team_2_score_allies = %d, team_2_score_axis = %d, team_2_points = team_2_points + %d,",
			"winner_id = %d",
			"WHERE id = %d LIMIT 1"
		}
		
		new pos = 0, len = sizeof(query_parts)
	
		for(new i = 0; i < len; i++)
		{
			pos += copy(query_str[pos], 511-pos, query_parts[i])
			pos += copy(query_str[pos], 511-pos, " ")
		}
		
		format(query_str, 511, query_str,
			M_FINISHED,
			g_t1_final_score, t1_score_allies, t1_score_axis, g_t1_points_change,
			g_t2_final_score, t2_score_allies, t2_score_axis, g_t2_points_change,
			g_winner,
			g_match_id
		)
		
		if(!SQL_SimpleQuery(g_connection, query_str))
		{
			log_amx("Unable to save match")
		}
		
		if(g_winner)
		{
			update_elo(g_t1_id, g_t1_points_change)
			update_elo(g_t2_id, g_t2_points_change)
		}
		
		cleanup()
		
		// Mark server as not running in DB
		new ip[32], port[12]
	
		get_cvar_string("ip", ip, 15)
		get_cvar_string("port", port, 5)
		
		mysql_escape_string(ip, 31)
		mysql_escape_string(port, 11)
		
		formatex(query_str, 255, "UPDATE halflife_server SET state = 1 WHERE bind_ip = '%s' AND port = '%s' LIMIT 1", ip, port)
		
		new error[512]
			
		if(!SQL_SimpleQuery(g_connection, query_str, error, 511))
		{
			log_amx("Unable to mark server as not running")
			log_amx("SQL Error: %s", error)
		}
		
		set_task(30.0, "shutdown", TASK_SHUTDOWN)
	}
	
// 	if(get_cvar_num("dmp_demos") == 1 || get_cvar_num("dmp_demos") == 3)
// 		pov_stop()
}

public delayed_change()
{
	new cMap[32]
	get_mapname(cMap, 31)
	server_cmd("changelevel %s", cMap)
}

public pre_live()
{
	readystate = 0
	new time_till = read_data(1)
	set_task(float(time_till), "live", TASK_LIVE) 
	
	if(task_exists(TASK_TEAM_CHECK))
		remove_task(TASK_TEAM_CHECK)
	
	switch(get_pcvar_num(cvar_match))
	{
		case 0:
			set_pcvar_num(cvar_match, 1)
		case 1:
			set_pcvar_num(cvar_match, 2)
		default:{}
	}
	
	set_hudmessage(255, 255, 255, -1.0, 0.15, 0, 6.0, 2.5, 0.5, 2.0, 3)
	ShowSyncHudMsg(0, g_hudsync, "DoD.HaveFun.cz^nDay of Defeat Community")
	
	set_task(5.0, "pre_live2", TASK_PRELIVE2)
	
	client_print(0, print_chat, "%L", LANG_PLAYER, "MATCH_COUNTDOWN", HF_TAG)
	client_print(0, print_chat, "%L", LANG_PLAYER, "MATCH_COUNTDOWN2", HF_TAG)
	client_print(0, print_chat, "%L", LANG_PLAYER, "MATCH_COUNTDOWN", HF_TAG)
	client_print(0, print_chat, "%L", LANG_PLAYER, "MATCH_COUNTDOWN2", HF_TAG)
	
	new query_str[256]
	formatex(query_str, 255, "UPDATE league_match SET state = %d WHERE id = %d LIMIT 1", M_BEING_PLAYED, g_match_id)
	
	SQL_ThreadQuery(g_SqlTuple, "common_query_handle", query_str)

	return PLUGIN_CONTINUE
}

public pre_live2()
{
	if(g_require_demos)
	{
		if(g_hltv_present)
		{
			new record_cmd[38]
			formatex(record_cmd, 37, "record %s%s-%d", g_hltv_demo_prefix, g_internal_name, g_match_id)
			hltv_rcon_command(record_cmd)
			g_hltv_recording = 1
		} else if(!get_pcvar_num(cvar_pov_recording)) { // HLTV is not here, for some reason, all players will record demo then
			new players[32], count, date_time[32], cmd[101]
			
			get_time("%Y_%m_%d-%H_%M_%S", date_time, 31)
			formatex(cmd, 100, "record %s_%s", g_internal_name, date_time)
			get_players(players, count, "ch")
			
			for(new i = 0; i < count; i++)
			{
				client_cmd(players[i], "stop")
				client_cmd(players[i], cmd)
				client_print(players[i], print_chat, "%L", players[i], "RECORDING_POV", HF_TAG, g_internal_name, date_time)
			}
			
			set_pcvar_num(cvar_pov_recording, 1)
		}
	}
	
	new Float:holdtime = float(get_cvar_num("mp_clan_timer")) - 5.0
	
	new text[451]
	
	if(g_match_id)
	{
		formatex(text, 450, "%s^n%s vs %s^n%L", g_league_name, g_t1_name, g_t2_name,
			LANG_PLAYER, (get_pcvar_num(cvar_match) == 1) ? "FIRST_HALF" : "SECOND_HALF")
	} else {
		formatex(text, 450, "%L", LANG_PLAYER, (get_pcvar_num(cvar_match) == 1) ? "FIRST_HALF" : "SECOND_HALF")
	}
	
	set_hudmessage(255, 255, 255, -1.0, 0.15, 0, 6.0, holdtime, 0.5, 0.15, 3)
	ShowSyncHudMsg(0, g_hudsync, text)
	
	return PLUGIN_CONTINUE
}

//Countdown ends - LIVE!
public live()
{
// 	if(get_cvar_num("dmp_collectstats") == 1) // FIXME: Remove?
// 	{
// 		new players[32], maxplayers, authid[32], name[32], team, mapname[16]
// 		get_mapname(mapname, 15)
// 		get_players(players, maxplayers)
// 		log_message("=== Delete everything above this line for stats ===")
// 		log_message("Loading map ^"%s^"", mapname)
// 		log_message("Stared map ^"%s^" (CRC ^"12345678^")", mapname) //Fixed! Changed 'starting map' to 'started map'
// 		
// 		for(new i=0; i < maxplayers; ++i)
// 		{
// 			get_user_name(players[i], name, 31)
// 			get_user_authid(players[i], authid, 31)
// 			team = get_user_team(players[i])
// 			log_message("^"%s<%d><%s><>^" entered the game", name, get_user_userid(players[i]), authid)
// 			if(team == 1)
// 			{
// 				log_message("^"%s<%d><%s><>^" joined team ^"Allies^"", name, get_user_userid(players[i]), authid)
// 				log_message("^"%s<%d><%s><Allies>^" changed role to ^"%s^"", name, get_user_userid(players[i]), authid, log_classnames[dod_get_next_class(players[i])])
// 			} else if(team == 2) {
// 				log_message("^"%s<%d><%s><>^" joined team ^"Axis^"", name, get_user_userid(players[i]), authid)
// 				log_message("^"%s<%d><%s><Axis>^" changed role to ^"%s^"", name, get_user_userid(players[i]), authid, log_classnames[dod_get_next_class(players[i])])
// 			}
// 			else
// 				log_message("^"%s<%d><%s><>^" joined team ^"Spectators^"", name, get_user_userid(players[i]), authid)
// 			}
// 			
// 			set_cvar_num("dodstats_pause",0)
// 	}
	
	set_task(3.0, "post_live_hf", TASK_POST_LIVE_HF)
	set_task(10.0, "post_live", TASK_POST_LIVE)
	
	return PLUGIN_CONTINUE
}

public post_live_hf()
{
	set_hudmessage(255, 0, 0, -1.0, 0.15, 0, 6.0, 2.0, 1.0, 1.0, 3)
	ShowSyncHudMsg(0, g_hudsync, "%L", LANG_PLAYER, "GL_HF")
}

public post_live()
{
	set_cvar_num("mp_timelimit", g_match_id ? g_map_half_timelimit : 15)
	
	g_t1_round_team = get_game_team(g_t1_id)
	g_t2_round_team = get_game_team(g_t2_id)
	
	return PLUGIN_CONTINUE
}

public print_results()
{
	if(!g_match_id)
	{
		client_print(0, print_chat, "%L", LANG_PLAYER, "EOM_SCORES", HF_TAG, 
			"Allies/Axis", g_t1_final_score, "Axis/Allies", g_t2_final_score
		)
		
		if(g_winner)
			client_print(0, print_chat, "%L", LANG_PLAYER, "EOM_WINNER", HF_TAG, (g_winner == ALLIES) ? "Allies/Axis" : "Axis/Allies")
		else
			client_print(0, print_chat, "%L", LANG_PLAYER, "EOM_WINNER_TIE", HF_TAG)
		
		return
	}
	
	client_print(0, print_chat, "%L", LANG_PLAYER, "EOM_SCORES_POINTS",
		HF_TAG, g_t1_name, g_t1_final_score, g_t1_points_change > 0 ? "+" : "", g_t1_points_change,
		g_t2_name, g_t2_final_score, g_t2_points_change > 0 ? "+" : "", g_t2_points_change
	)
	
	if(g_winner)
		client_print(0, print_chat, "%L", LANG_PLAYER, "EOM_WINNER", HF_TAG, (g_winner == g_t1_id) ? g_t1_name : g_t2_name)
	else
		client_print(0, print_chat, "%L", LANG_PLAYER, "EOM_WINNER_TIE", HF_TAG)
}

public ready_loop()
{
	if(readystate == 1)
	{
		if(alliesready == 1)
			client_print(0, print_chat, "%L", LANG_PLAYER, "READY_LOOP1", HF_TAG)
		else if(axisready == 1)
			client_print(0, print_chat, "%L", LANG_PLAYER, "READY_LOOP2", HF_TAG)
		else
			client_print(0, print_chat, "%L", LANG_PLAYER, "READY_LOOP3", HF_TAG)
		
		client_print(0, print_chat, "%L", LANG_PLAYER, "SAY_READY", HF_TAG)
		client_print(0, print_chat, "%L", LANG_PLAYER, "SAY_NOTREADY", HF_TAG)
	}
	
	set_task(40.0, "ready_loop", 0)
}

public shutdown()
{
	 server_cmd("quit")
}

public fm_keyvalue(entid, handle)
{
	if( pev_valid(entid) )
	{
		new classname[100], key[100], value[100]
		
		get_kvd(handle, KV_ClassName, classname, 99)
		get_kvd(handle, KV_KeyName, key, 99)
		get_kvd(handle, KV_Value, value, 99)
		
		if( equali(classname, "dod_control_point") && equali(key, "point_index") ) {
			g_cps[g_cp_count] = str_to_num(value)
			
			g_cp_count++
		}
	}
}

public common_query_handle(failstate, Handle:query, error[], errno, data[], datasize)
{
	if(failstate == TQUERY_CONNECT_FAILED)
       		log_amx("Could not connect to SQL database.")
	else if(failstate == TQUERY_QUERY_FAILED)
	{
		log_amx("Error on query: %s", error)
		log_amx("Query failed.")
	}
	
	if(errno)
		return log_amx("Error on query: %s", error)
	
	return PLUGIN_CONTINUE
}

public hltv_check()
{
	if(!g_hltv_present)
	{
		new cmd[128], ip[16], port[6]
		
		get_cvar_string("ip", ip, 15)
		get_cvar_string("port", port, 5)
		
		formatex(cmd, 127, "connect %s:%s", ip, port)
		hltv_rcon_command(cmd)
	}
}

public hltv_stop_recording()
{
	hltv_rcon_command("stoprecording")
}

public hltv_rcon_command(hltv_command[])
{
	// Declare variables
	new socket_address		// Contains the socket address of the hltv server 
	new socket_error = 0	// Contains the error code of the socket connection
	new send[256]			// Contains the send socket command	
		
	// Connect to the HLTV Proxy
	socket_address = socket_open(g_hltv_ip, g_hltv_port, SOCKET_UDP, socket_error)
		
	if (socket_error != 0)
		return log_amx("HLTV connection failure...", socket_error)
		
	// Send challenge rcon and receive response
	// Do NOT add spaces after the commas, you get an error about invalid function call
	if(equali(g_hltv_challenge, ""))
	{
		if(!g_challenging_rcon)
		{
			setc(send, 4, 0xff)
			copy(send[4], 255, "challenge rcon")
			setc(send[28], 1, '^n')
			
			socket_send2(socket_address, send, 255)	
      
			set_task(2.0, "hltv_challenge_receive", socket_address)
			g_challenging_rcon = true
		}
		set_task(4.2, "hltv_rcon_command", 0, hltv_command, strlen(hltv_command))	
	} else {
		replace(g_hltv_challenge, 255, "^n", "")
		
		// Set rcon command
		setc(send, 255, 0x00)
		setc(send, 4, 0xff)
		
		log_amx("hltv_command: %s", hltv_command)
		formatex(send[4], 255, "rcon %s ^"%s^" %s ^n", g_hltv_challenge, g_hltv_password, hltv_command)
		log_amx("sending: %s", send)
		socket_send2(socket_address, send, 255)
		socket_close(socket_address)
				
// 		switch(hltv_command[0])
// 		{
// 			case 'r':
// 			{
// 				if(g_show_time > -2)
// 					set_task(1.0, "hltv_show_time", TIMER_TASK, _, _, "b")
// 				g_hltv_recording = 4
// 				set_pcvar_num(g_recording_cvar, 1)	
// 			}
// 			case 's':		
// 			{
// 				remove_task(str_to_num(g_hltv_challenge))
// 				g_hltv_recording = 0
// 				set_pcvar_num(g_recording_cvar, 0)
// 			}
// 		}
			
		socket_close(socket_address)
	}
	return PLUGIN_CONTINUE
}

public hltv_challenge_receive(socket_address)	
{
	if(socket_change(socket_address))
	{
		if(task_exists(socket_address))
			remove_task(socket_address)
		new receive[255]	
		socket_recv(socket_address, receive, 255)
		copy(g_hltv_challenge, 12, receive[19])
		set_task(20.0, "hltv_freehandle_challenge")
		g_challenging_rcon = false
	}
	else
	{
		new send[255], socket_error = 0
		socket_close(socket_address)
		socket_address = socket_open(g_hltv_ip, g_hltv_port, SOCKET_UDP, socket_error)
		if(socket_error > 0)
			log_amx("HLTV not responding...")
		else
		{
			if(!task_exists(socket_address))
			{
				setc(send, 4, 0xff)
				copy(send[4], 255, "challenge rcon")
				setc(send[28], 1, '^n')

				socket_send2(socket_address, send, 255)	
				set_task(1.0, "hltv_challenge_receive", socket_address)
			}
		}
	} 
}

public hltv_freehandle_challenge()
{
	g_hltv_challenge = ""
}

stock fix_flags_order(values[], len = 0)
{
	new cp, backup[10]
	
	copy(backup, 9, values)
	
	for(new i = 0; i < len; i++)
	{
		cp = g_cps[i] - 1
		
		if(cp >= g_cp_count)
		{
			log_amx("Error fixing flag order")
			return
		}
		
		values[cp] = backup[i]
	}
}

stock add_player_history(id, event)
{
	new query_str[256], player[3]
	
	ArrayGetArray(g_valid_players, g_players[id][INDEX], player)
		
	if(!player[MATCH_USER_ID])
		return
	
	formatex(query_str, 255, "INSERT INTO league_matchuserhistory (match_user_id, event, date) VALUES (%d, %d, CONVERT_TZ(NOW(), '%s', 'UTC'))",
		player[MATCH_USER_ID], event, HF_TZ
	)
	
	SQL_ThreadQuery(g_SqlTuple, "common_query_handle", query_str)
}

stock add_penalty(team_id, penalty)
{
	new query_str[256], penalty_str[51]
	
	switch(penalty)
	{
		case PENALTY_NOT_PRESENT:
		{
			copy(penalty_str, 50, "Not present.")
		}
		default:
		{
			log_amx("Unknown penalty %d", penalty)
			return
		}
	}
	
	formatex(query_str, 255, "INSERT INTO league_penalty (match_id, team_id, admin_id, date, points, reason, included) VALUES (%d, %d, NULL, CONVERT_TZ(NOW(), '%s', 'UTC'), %d, '%s', 1)",
		g_match_id, team_id, HF_TZ, g_penalties[penalty], penalty_str
	)
	
	if(!SQL_SimpleQuery(g_connection, query_str))
	{
		log_amx("Unable to save penalty")
	}
	
	formatex(query_str, 255, "UPDATE league_team SET points = points - %d WHERE id = %d LIMIT 1",
		g_penalties[penalty], team_id
	)
	
	if(!SQL_SimpleQuery(g_connection, query_str))
	{
		log_amx("Unable to save team's points")
	}
}

stock get_game_team(team_id)
{
	new players[32], cnt = 0, player[3]
	get_players(players, cnt, "ch")
	
	for(new i = 0; i < cnt; i++)
	{
		ArrayGetArray(g_valid_players, g_players[players[i]][INDEX], player)
		
		if( player[TEAM_ID] == team_id )
			return get_user_team(players[i])
	}
	
	return -1
}

stock update_elo(team_id, elo)
{
	new query_str[128], error[512]
	
	formatex(query_str, 127, "UPDATE league_team SET points = points + %d WHERE id = %d LIMIT 1",
		elo, team_id
	)
	
	if(!SQL_SimpleQuery(g_connection, query_str, error, 511))
	{
		log_amx("Failed to update team elo: team_id=%d, elo=%d", team_id, elo)
		log_amx("SQL Error: %s", error)
	}
}

stock cleanup()
{
	set_pcvar_num(cvar_t1_score, 0)
	set_pcvar_num(cvar_t2_score, 0)
	set_pcvar_num(cvar_score_allies, 0)
	set_pcvar_num(cvar_score_axis, 0)
	set_pcvar_num(cvar_pause_t1, 0)
	set_pcvar_num(cvar_pause_t2, 0)
	set_pcvar_num(cvar_pov_recording, 0)
	set_pcvar_num(cvar_match, 0)
}

stock calc_elo(t_1, t_2, s_1, s_2)
{
	new Float:c2 = 400.0
	new Float:c1 = 50.0
	
	new Float:w, Float:e
	
	// First team
	w = (float(s_1) / (float(s_1 + s_2) / 100.0)) / 100.0
	//server_print("W: %.2f", w)
	
	e = 1.0 / (1.0 + (floatpower(10.0, (-float((t_1 - t_2)) / c2))))
	//server_print("E: %.2f", e)
	
	g_t1_points_change = floatround(c1 * ( w - e ))
	//server_print("Nove ELO - TEAM 1: %d", elo)
	
	// Second team
	w = (float(s_2) / (float(s_1 + s_2) / 100.0)) / 100.0
	//server_print("W: %.2f", w)
	
	e = 1.0 / (1.0 + floatpower(10.0, (-float(t_2 - t_1) / c2)))
	//server_print("E: %.2f", e)
	
	g_t2_points_change = floatround(c1 * ( w - e ))
	//server_print("Nove ELO - TEAM 2: %d", elo);
}

stock check_execute(Handle:query)
{
	if( !SQL_Execute(query) )
	{
		new error[512]
		SQL_QueryError(query, error, 511)
		log_amx("SQL error: %s", error)
		
		if(equal(error, "MySQL server has gone away"))
		{
			g_loaded = 0
			SQL_FreeHandle(g_connection)
			g_connection = Empty_Handle
			
			if(!task_exists(TASK_CONFIGURE))
				set_task(15.0, "configure", TASK_CONFIGURE)
		}
		
		SQL_FreeHandle(query)
		
		return 0
	}
	
	return 1
}
