/**
 * Copyright (C) 2010-2013 Jakub Skokan <aither@havefun.cz>
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3
 * as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <amxmodx>
#include <sqlx>
#include <hf_api>
#include <hf_dod_match>

#define MATCHES_COUNT 5
#define FETCH_FREQUENCY 300.0
#define AD_FREQUENCY 90.0

enum {
	TASK_CONFIGURE=40,
	TASK_FETCH=50,
	TASK_ADVERTISE=60,
}

enum _:Match {
	LEAGUE_NAME[101],
	LEAGUE_TEAM_SIZE,
	TEAM_NAME[101],
	TEAM_POINTS,
	MAP[31],
	DATE[21],
}

new Handle:g_SqlTuple

new g_query_str[512]
new g_matches[MATCHES_COUNT][Match]
new g_matches_cnt = 0
new g_advertise_step = 0

public plugin_init()
{
	register_plugin("HF DoD Match Advertiser", HF_VERSION, "aither")
	register_dictionary("hf_dod_match_advertiser.txt")
}

public plugin_cfg()
{
	set_task(0.25, "configure", TASK_CONFIGURE)
}

public plugin_end()
{
	SQL_FreeHandle(g_SqlTuple)
}

public configure()
{
	new host[64], user[64], pass[64], db[64]
	get_cvar_string("amx_sql_host", host, 63)
	get_cvar_string("amx_sql_user", user, 63)
	get_cvar_string("amx_sql_pass", pass, 63)
	get_cvar_string("amx_sql_db", db, 63)
	
	g_SqlTuple = SQL_MakeDbTuple(host, user, pass, db)
	
	new query_parts[][] = {
		"SELECT l.name AS league_name, l.team_size, t.name_ascii AS team_name, t.points, map.name AS map, DATE_FORMAT(r.date, '%%d.%%m. %%H:%%i') AS `date`",
		"FROM (((league_match m INNER JOIN league_reservation r ON r.id = m.reservation_id)",
		"INNER JOIN league_map map ON map.id = r.map_id)",
		"INNER JOIN league_team t ON t.id = m.team_1_id)",
		"INNER JOIN league_league l ON l.id = t.league_id",
		"WHERE m.state = %d AND r.date >= CONVERT_TZ(NOW(), '%s', 'UTC')",
		"ORDER BY r.date LIMIT %d"
	}
	
	new len = sizeof(query_parts), pos = 0
	
	for(new i = 0; i < len; i++)
	{
		pos += copy(g_query_str[pos], 511-pos, query_parts[i])
		pos += copy(g_query_str[pos], 511-pos, " ")
	}
	
	format(g_query_str, 511, g_query_str, M_LOOKING, HF_TZ, MATCHES_COUNT)
	
	fetch_data()
	
	set_task(FETCH_FREQUENCY, "fetch_data", TASK_FETCH, "", 0, "b")
	
	return PLUGIN_CONTINUE
}

public fetch_data()
{
	SQL_ThreadQuery(g_SqlTuple, "fetch_data_process", g_query_str)
}

public fetch_data_process(failstate, Handle:query, error[], errno, data[], datasize)
{
	if(failstate == TQUERY_CONNECT_FAILED)
		log_amx("Could not connect to SQL database.")
	else if(failstate == TQUERY_QUERY_FAILED)
	{
		log_amx("Error on query: %s", error)
		log_amx("Query failed.")
	}
	
	if(errno)
		return log_amx("Error on query: %s", error)
	
	if(!SQL_NumResults(query))
	{
		if(task_exists(TASK_ADVERTISE))
			remove_task(TASK_ADVERTISE)
		
		return PLUGIN_CONTINUE
	}
	
	g_matches_cnt = 0
	
	while(SQL_MoreResults(query))
	{
		SQL_ReadResult(query, SQL_FieldNameToNum(query, "league_name"), g_matches[g_matches_cnt][LEAGUE_NAME], 100)
		g_matches[g_matches_cnt][LEAGUE_TEAM_SIZE] = SQL_ReadResult(query, SQL_FieldNameToNum(query, "team_size"))
		SQL_ReadResult(query, SQL_FieldNameToNum(query, "team_name"), g_matches[g_matches_cnt][TEAM_NAME], 100)
		g_matches[g_matches_cnt][TEAM_POINTS] = SQL_ReadResult(query, SQL_FieldNameToNum(query, "points"))
		SQL_ReadResult(query, SQL_FieldNameToNum(query, "map"), g_matches[g_matches_cnt][MAP], 30)
		SQL_ReadResult(query, SQL_FieldNameToNum(query, "date"), g_matches[g_matches_cnt][DATE], 20)
		
		g_matches_cnt++
		
		SQL_NextRow(query)
	}
	
	g_advertise_step = 0
	advertise()
	set_task(AD_FREQUENCY, "advertise", TASK_ADVERTISE, "", 0, "b")
	
	return PLUGIN_CONTINUE
}

public advertise()
{
	if(!g_matches_cnt)
		return
	
	if(g_advertise_step >= g_matches_cnt)
		g_advertise_step = 0
	
	client_print(0, print_chat, "%L", LANG_PLAYER, "AD",
		g_matches[g_advertise_step][LEAGUE_NAME], g_matches[g_advertise_step][LEAGUE_TEAM_SIZE], g_matches[g_advertise_step][LEAGUE_TEAM_SIZE],
		g_matches[g_advertise_step][TEAM_NAME], g_matches[g_advertise_step][TEAM_POINTS], g_matches[g_advertise_step][MAP], g_matches[g_advertise_step][DATE]
	)
	
	client_print(0, print_chat, "%L", LANG_PLAYER, "INFO", HF_WEB_URL)
	
	g_advertise_step++
}
